// MainWnd.cpp : implementation file
//

#include "stdafx.h"
#include "SysTrayPlay.h"
#include "MainWnd.h"
#include "EqualizerDlg.h"
#include "PlaylistDlg.h"
#include "AboutDlg.h"
#include "SeekDialog.h"
#include "MP3InfoDlg.h"
#include "SettingsDialog.h"
#include "UrlDlg.h"
#include "AutDlg.h"
#include "MainHelper.h"
#include "WAEmuWnd.h"
#include "AutoUpdater.h"
#include "PlaylistManager.h"
#include "In/StreamManager.h"

#include "out/MultiDllOutput.h"
#include "In/MultiDllInput.h"
#include "Gen/MultiDllGen.h"

#include "In/Args.h"
#include "In/stream.h"
#include "In/frame.h"

#include "Volume.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//Redfinition pour Visual C++ 6.0
#ifndef MN_GETHMENU
#define MN_GETHMENU 					0x01E1
#endif

CStringArray genres;

Console* pConsole;
CWAEmuWnd* WAEmuWnd;
CMainWnd* pMainFrame;
extern CSysTrayPlayApp theApp;
extern CAboutDlg* AboutDialog;
CEqualizerDlg* EqualizerDlg = NULL;
CPlaylistDlg* PlaylistDlg = NULL;
CMP3InfoDlg* MP3InfoDlg = NULL;
CSettingsDialog* SettingsDialog = NULL;
CSeekDialog* SeekDialog = NULL;
CAutDlg* AutDlg = NULL;

CVolume* pVolume = NULL;

Helper m_Helper;

extern HANDLE hAutomationThread;
PlaylistManager m_PlaylistManager;
extern MPArgs Args;
unsigned long rnd;

CMultiDllInput* stdIn = NULL;
CMultiDllOutput* stdOut = NULL;
CMultiDllGen* stdGen = NULL;

CImageList m_imgList;

// TAGS STUFF

struct simple_frame
{
	long lsf;
	long lay;
	long error_protection;
	long bitrate_index;
	long sampling_frequency;
	long mpeg25;
	long stereo;
};

static bool simple_decode_header(unsigned long newhead, struct simple_frame* pfr)
{
	// AAAAAAAA AAABBCCD EEEEFFGH IIJJKLMM

	// AAAAAAAA AAA Frame sync (all bits must be set)
	// 31-21 bits
	if ((newhead & 0xffe00000) != 0xffe00000)
		return false;

	// CC  Layer description
	// 18-17 bits
	//    00 - reserved
	//     01 - Layer III
	//     10 - Layer II
	//     11 - Layer I
	if (!((newhead >> 17) & 0x3))
		return false;

	/* EEEE Bitrate index
	   15-12 bits
		   bits V1,L1    V1,L2   V1,L3   V2,L1   V2, L2 & L3 
		   0000 free	 free    free    free    free 
		   0001 32     32   	 32 	   32   	 8 
		   0010 64     48   	 40 	   48   	 16 
		   0011 96     56   	 48 	   56   	 24 
		   0100 128 	 64 	   56   	 64 	   32 
		   0101 160 	 80 	   64   	 80 	   40 
		   0110 192 	 96 	   80   	 96 	   48 
		   0111 224 	 112	   96   	 112	   56 
		   1000 256 	 128	   112  	 128	   64 
		   1001 288 	 160	   128  	 144	   80 
		   1010 320 	 192	   160  	 160	   96 
		   1011 352 	 224	   192  	 176	   112 
		   1100 384 	 256	   224  	 192	   128 
		   1101 416 	 320	   256  	 224	   144 
		   1110 448 	 384	   320  	 256	   160 
		   1111 bad 	 bad	   bad  	 bad	   bad 
		   NOTES: All values are in kbps
		   V1 - MPEG Version 1
		   V2 - MPEG Version 2 and Version 2.5
		   L1 - Layer I
		   L2 - Layer II
		   L3 - Layer III
		   
		   "free" means free format. The free bitrate must remain constant, 
		   an must be lower than the maximum allowed bitrate. Decoders are 
		   not required to support decoding of free bitrate streams.
		   "bad" means that the value is unallowed. 
		   
		   MPEG files may feature variable bitrate (VBR). Each frame may 
		   then be created with a different bitrate. It may be used in all 
		   layers. Layer III decoders must support this method. Layer I & II 
		   decoders may support it. 
		   
		   For Layer II there are some combinations of bitrate and mode which 
		   are not allowed. Here is a list of allowed combinations. 
		   bitrate single channel  stereo  intensity stereo 	 dual channel 
		   free 	   yes  		 yes			   yes  			 yes 
		   32   		 yes		   no   			 no 			   no
		   48   		 yes		   no   			 no 			   no 
		   56   		 yes		   no   			 no 			   no 
		   64   		 yes		   yes  			 yes			   yes 
		   80   		 yes		   no   			 no 			   no 
		   96   		 yes		   yes  			 yes			   yes 
		   112  		 yes		   yes  			 yes			   yes 
		   128  		 yes		   yes  			 yes			   yes 
		   160  		 yes		   yes  			 yes			   yes 
		   192  		 yes		   yes  			 yes			   yes 
		   224  		 no 		   yes  			 yes			   yes 
		   256  		 no 		   yes  			 yes			   yes 
		   320  		 no 		   yes  			 yes			   yes 
		   384  		 no 		   yes  			 yes			   yes 
	*/
	if (((newhead >> 12) & 0xf) == 0xf)
		return false;

	// FF      Sampling rate frequency index 
	// 11-10 bits
	//  		 bits    MPEG1  	 MPEG2  	 MPEG2.5 
	//  		 00 	   44100 Hz 	 22050 Hz      11025 Hz 
	//  		 01 	   48000 Hz 	 24000 Hz      12000 Hz 
	//  		 10 	   32000 Hz 	 16000 Hz      8000 Hz 
	//  		 11 	   reserv.     reserv.     reserv. 

	if (((newhead >> 10) & 0x3) == 0x3)
		return false;

	// BB MPEG Audio version ID
	// 20-19 bits
	//  	 00 - MPEG Version 2.5 (later extension of MPEG 2)
	//  	 01 - reserved
	//  	 10 - MPEG Version 2 (ISO/IEC 13818-3)
	//  	 11 - MPEG Version 1 (ISO/IEC 11172-3) 
	//  	 Note: MPEG Version 2.5 was added lately to the MPEG 2 standard. 
	//  	 It is an extension used for very low bitrate files, allowing 
	//  	 the use of lower sampling frequencies. If your decoder does not 
	//  	 support this extension, it is recommended for you to use 12 bits 
	//  	 for synchronization instead of 11 bits.

	if (((newhead >> 19) & 0x3) == 0x1)
		return false;
	if (((newhead >> 19) & 0x3) == 0x0)
	{
		pfr->lsf = 1;
		pfr->mpeg25 = 1;
	}
	else
	{
		pfr->lsf = ((newhead >> 19) & 0x1);
		pfr->mpeg25 = 0;
	}

	pfr->lay = 4 - ((newhead >> 17) & 3);

	// II Channel Mode
	// 7-6 bits
	//   00 - Stereo
	//   01 - Joint stereo (Stereo)
	//   10 - Dual channel (2 mono channels)
	//   11 - Single channel (Mono)
	//
	//   Note: Dual channel files are made of two independant mono channel. 
	//   Each one uses exactly half the bitrate of the file. Most decoders 
	//   output them as stereo, but it might not always be the case.
	//   One example of use would be some speech in two different languages 
	//   carried in the same bitstream, and then an appropriate decoder would 
	//   decode only the choosen language
	if (pfr->mpeg25 == 1)
		pfr->sampling_frequency = 6 + ((newhead >> 10) & 0x3);
	else
		pfr->sampling_frequency = ((newhead >> 10) & 0x3) + (pfr->lsf * 3);

	// D Protection bit
	// 16 bits
	//  	 0 - Protected by CRC (16bit CRC follows header)
	//  	 1 - Not protected
	pfr->error_protection = (~(newhead >> 16) & 0x1);

	pfr->bitrate_index = ((newhead >> 12) & 0xf);

	pfr->stereo = (((newhead >> 6) & 0x3) == MPG_MD_MONO) ? 1 : 2;

	return true;
}

// END OF TAGS STUFF
static int __stdcall BrowseCallbackProc(HWND hwnd, UINT uMsg, LPARAM lParam, LPARAM lpData)
{
	CString dir;
	dir = pMainFrame->InitDir;
	if (dir.Right(1) == "\\")
		dir = dir.Left(dir.GetLength() - 1);

	if (uMsg == BFFM_INITIALIZED)
		::SendMessage(hwnd, BFFM_SETSELECTION, TRUE, (LPARAM)(LPCTSTR)dir);
	return 0;
}

/////////////////////////////////////////////////////////////////////////////
// CMainWnd

IMPLEMENT_DYNCREATE(CMainWnd, CFrameWnd);

CMainWnd::CMainWnd() : CFrameWnd()
{
	mainMenu = new CMenu();
	Created = false;
}

CMainWnd::~CMainWnd()
{
	delete stdIn;
	delete stdOut;
	delete stdGen;
	delete mainMenu;
	pMainFrame = NULL;
}

BEGIN_MESSAGE_MAP(CMainWnd, CFrameWnd)
	ON_COMMAND(ID_POPUPMENU_EXIT, OnPopupMenuExit)
	ON_COMMAND(ID_POPUPMENU_OPEN_FILE, OnPopupMenuOpenFile)
	ON_COMMAND(ID_POPUPMENU_ADD_FILE, OnPopupMenuAddFile)
	ON_COMMAND(ID_POPUPMENU_PLAY, OnPopupMenuPlay)
	ON_COMMAND(ID_POPUPMENU_PAUSE, OnPopupMenuPause)
	ON_COMMAND(ID_POPUPMENU_STOP, OnPopupMenuStop)
	ON_COMMAND(ID_POPUPMENU_ABOUT, OnPopupMenuAbout)
	ON_COMMAND(ID_POPUPMENU_NEXT, OnPopupMenuNext)
	ON_COMMAND(ID_POPUPMENU_PREVIOUS, OnPopupMenuPrevious)
	ON_COMMAND(ID_POPUPMENU_EQUALIZER, OnPopupMenuEqualizer)
	ON_COMMAND(ID_POPUPMENU_PLAYLISTEDITOR, OnPopupMenuPlaylistEditor)
	ON_COMMAND(ID_POPUPMENU_SEEK, OnPopupMenuSeek)
	ON_COMMAND(ID_POPUPMENU_OPENFOLDER, OnPopupMenuOpenFolder)
	ON_COMMAND(ID_POPUPMENU_ADDFOLDER, OnPopupMenuAddFolder)
	ON_COMMAND(ID_POPUPMENU_FILEINFO, OnPopupMenuFileInfo)
	ON_UPDATE_COMMAND_UI(ID_POPUPMENU_FILEINFO, OnUpdatePopupMenuFileInfo)
	ON_UPDATE_COMMAND_UI(ID_POPUPMENU_SETTINGS_REPEAT, OnUpdatePopupMenuSettingsRepeat)
	ON_COMMAND(ID_POPUPMENU_SETTINGS_REPEATTRACK, OnPopupMenuSettingsRepeatTrack)
	ON_UPDATE_COMMAND_UI(ID_POPUPMENU_SETTINGS_REPEATTRACK, OnUpdatePopupMenuSettingsRepeatTrack)
	ON_WM_ENDSESSION()
	ON_COMMAND(ID_POPUPMENU_SETTINGS_REPEAT, OnPopupMenuSettingsRepeat)
	ON_COMMAND(ID_POPUPMENU_SETTINGS_ADVANCED, OnPopupMenuSettingsAdvanced)
	ON_COMMAND(ID_POPUPMENU_SETTINGS_SHUFFLEMODE, OnPopupMenuSettingsShuffleMode)
	ON_UPDATE_COMMAND_UI(ID_POPUPMENU_SETTINGS_SHUFFLEMODE, OnUpdatePopupMenuSettingsShuffleMode)
	ON_WM_COPYDATA()
	ON_WM_CREATE()
	ON_UPDATE_COMMAND_UI(ID_POPUPMENU_EQUALIZER, OnUpdatePopupMenuEqualizer)
	ON_UPDATE_COMMAND_UI(ID_POPUPMENU_PLAYLISTEDITOR, OnUpdatePopupMenuPlaylistEditor)
	ON_UPDATE_COMMAND_UI(ID_POPUPMENU_SEEK, OnUpdatePopupMenuSeek)
	ON_WM_CLOSE()
	ON_COMMAND(ID_POPUPMENU_ELAPSEDTIME, OnPopupMenuElapsedTime)
	ON_UPDATE_COMMAND_UI(ID_POPUPMENU_ELAPSEDTIME, OnUpdatePopupMenuElapsedTime)
	ON_COMMAND(ID_POPUPMENU_REMAININGTIME, OnPopupMenuRemainingTime)
	ON_UPDATE_COMMAND_UI(ID_POPUPMENU_REMAININGTIME, OnUpdatePopupMenuRemainingTime)
	ON_COMMAND(ID_POPUPMENU_SETTINGS_RANDOMLYSELECTFIRSTTRACK, OnPopupMenuSettingsRandomlySelectFirstTrack)
	ON_UPDATE_COMMAND_UI(ID_POPUPMENU_SETTINGS_RANDOMLYSELECTFIRSTTRACK, OnUpdatePopupMenuSettingsRandomlySelectFirstTrack)
	ON_COMMAND(ID_POPUPMENU_FILENAME, OnPopupMenuFilename)
	ON_UPDATE_COMMAND_UI(ID_POPUPMENU_FILENAME, OnUpdatePopupMenuFilename)
	ON_COMMAND(ID_POPUPMENU_TITLE, OnPopupMenuTitle)
	ON_UPDATE_COMMAND_UI(ID_POPUPMENU_TITLE, OnUpdatePopupMenuTitle)
	ON_COMMAND(ID_POPUPMENU_ARTISTTITLE, OnPopupMenuArtistTitle)
	ON_UPDATE_COMMAND_UI(ID_POPUPMENU_ARTISTTITLE, OnUpdatePopupMenuArtistTitle)
	ON_COMMAND(ID_POPUPMENU_OPEN_URL, OnPopupMenuOpenUrl)
	ON_COMMAND(ID_POPUPMENU_SETTINGS_AFTERENDOFTHEPLAYBACK_EXITSTP, OnPopupMenuSettingsAfterEndOfThePlaybackExitStp)
	ON_UPDATE_COMMAND_UI(ID_POPUPMENU_SETTINGS_AFTERENDOFTHEPLAYBACK_EXITSTP, OnUpdatePopupMenuSettingsAfterEndOfThePlaybackExitStp)
	ON_COMMAND(ID_POPUPMENU_SETTINGS_AFTERENDOFTHEPLAYBACK_TURNOFFTHECOMPUTER, OnPopupMenuSettingsAfterEndOfThePlaybackTurnOffTheComputer)
	ON_UPDATE_COMMAND_UI(ID_POPUPMENU_SETTINGS_AFTERENDOFTHEPLAYBACK_TURNOFFTHECOMPUTER, OnUpdatePopupMenuSettingsAfterEndOfThePlaybackTurnOffComputer)
	ON_WM_TIMER()
	ON_COMMAND(ID_POPUPMENU_DOCKTOACTIVEWINDOW, OnPopupMenuDockToActiveWindow)
	ON_UPDATE_COMMAND_UI(ID_POPUPMENU_DOCKTOACTIVEWINDOW, OnUpdatePopupMenuDockToActiveWindow)
	ON_COMMAND(ID_POPUPMENU_SHOW_TOTALTRACKTIME, OnPopupMenuShowTotalTrackTime)
	ON_UPDATE_COMMAND_UI(ID_POPUPMENU_SHOW_TOTALTRACKTIME, OnUpdatePopupMenuShowTotalTrackTime)
	ON_COMMAND(ID_POPUPMENU_SETTINGS_STOPAFTEREACHALBUM, OnPopupMenuSettingsStopAfterEachAlbum)
	ON_UPDATE_COMMAND_UI(ID_POPUPMENU_SETTINGS_STOPAFTEREACHALBUM, OnUpdatePopupMenuSettingsStopAfterEachAlbum)
	ON_COMMAND(ID_AUTOMATION_SETUP, OnAutomationSetup)
	ON_COMMAND(ID_AUTOMATION_PERFORMNOW, OnAutomationPerformNow)
	ON_COMMAND_RANGE(IDM_RECENT_FILES,IDM_RECENT_FILES+9,OnPopupMenuOpenRecent)
	ON_COMMAND_RANGE(IDM_TRACKS,IDM_TRACKS+49999,OnTrackMenu)
	ON_COMMAND_RANGE(IDM_DRIVES,IDM_DRIVES+25,OnDrivesMenu)
	//TODO ON_COMMAND(ID_FILE_ADDFILE, OnFileAddFile)
	ON_MESSAGE(WM_PLAYOVER,OnPostPlayerOver)
	ON_MESSAGE(WM_ICON_NOTIFY,IconNotify)
	ON_WM_MEASUREITEM()
	ON_WM_DRAWITEM()
	ON_WM_INITMENUPOPUP()
	ON_MENUXP_MESSAGES()
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMainWnd message handlers

void CMainWnd::AddRecent(CString fdn)
{
	if (fdn[fdn.GetLength() - 1] == '\\')
		fdn = fdn.Left(fdn.GetLength() - 1);
	if (fdn[1] == ':' && fdn.GetLength() == 2)
		return;
	int i;
	for (i = 0; i <= 9; i++)
	{
		if (fdn.CompareNoCase(Recent[i]) == 0)
		{
			for (int j = i; j > 0; j--)
				Recent[j] = Recent[j - 1];
			Recent[0] = fdn;
			return;
		}
	}
	for (i = 9; i > 0; i--)
		Recent[i] = Recent[i - 1];
	Recent[0] = fdn;
}

void CMainWnd::BuildAlbumMenu(CMenu* m)
{
	UINT start, finish;
	UINT i, cnt = -1;
	GetAlbumRange(&start, &finish, true);
	CString folder = m_PlaylistManager.GetPlaylist(Current);
	folder = m_PlaylistManager.GetPlaylist(start).Left(m_PlaylistManager.GetPlaylist(start).ReverseFind('\\'));
	if (folder.Find('\\', 0) != -1)
	{
		folder = folder.Right(folder.GetLength() - folder.ReverseFind('\\') - 1);
		m->AppendMenu(MF_STRING, 65535, folder);
		m->AppendMenu(MF_SEPARATOR);
	}
	for (i = start; i <= finish; i++)
	{
		CString buf;
		buf.Format(_T("%d|%s"), i, m_PlaylistManager.GetPlaylist(i));
		cnt++;
		if (TPC && cnt % TPC == 0 && cnt > 0)
		{
			m->AppendMenu(MF_STRING | MF_MENUBARBREAK, IDM_TRACKS - 1, _T(""));
			m->AppendMenu(MF_SEPARATOR);
		}
		m->AppendMenu(MF_STRING, IDM_TRACKS + i, buf);

		if (i == Current)
			m->SetDefaultItem(m->GetMenuItemCount() - 1, TRUE);
	}
	//On comble par des blancs
	if (TPC && cnt / TPC > 0)
		for (UINT i = cnt + 1; i % TPC != 0; i++)
			m->AppendMenu(MF_STRING, IDM_TRACKS - 1, _T(""));
}

void CMainWnd::BuildMenu(CMenu* m)
{
	UINT i = 0, j = 0;
	int sl;
	CString buf;
	int cnt = -1;

	do
	{
		if (m_PlaylistManager.GetPlaylist(i).Left(2) == "\\\\")
			buf = m_PlaylistManager.GetPlaylist(i).Left(m_PlaylistManager.GetPlaylist(i).Find(_T("\\"), 2) + 1);
		else
			buf = m_PlaylistManager.GetPlaylist(i).Left(m_PlaylistManager.GetPlaylist(i).FindOneOf(_T("\\/")) + 1);
		do
			i++;
		while (i < (UINT)m_PlaylistManager.GetSize() && m_PlaylistManager.GetPlaylist(i).Left(buf.GetLength()).CompareNoCase(buf) == 0);
		sl = GetStartLevel(j, i);
		if (sl > 0 && !(j == 0 && i == m_PlaylistManager.GetSize()))
		{
			if (sl == GetMaxLevel(j, i))
				sl--;
		}
		BuildMenuLevel(m, sl, j, i, "", &cnt);
		j = i;
	}
	while (i < (UINT)m_PlaylistManager.GetSize());
}

int CMainWnd::BuildMenuLevel(CMenu* m, int l, UINT start, UINT finish, CString cur, int* counter)
{
	int cl = cur.GetLength();
	int cnt = -1;
	if (counter != NULL)
		cnt = *counter;
	CString buf;
	for (UINT i = start; i < finish; i++)
	{
		if (m_PlaylistManager.GetPlaylist(i).Left(cl).CompareNoCase(cur) != 0)
		{
			//On comble par des blancs
			if (TPC && cnt / TPC > 0)
				for (UINT j = cnt + 1; j % TPC != 0; j++)
					m->AppendMenu(MF_STRING, IDM_TRACKS - 1, _T(""));
			return i - 1;
		}

		if (ExtractLevel(m_PlaylistManager.GetPlaylist(i), l + 1, &buf))
		{
			HMENU nm = CreatePopupMenu();
			cnt++;
			m->AppendMenu(MF_POPUP | ((TPC && cnt % TPC == 0 && cnt > 0) ? MF_MENUBARBREAK : 0), (UINT)nm, ReplaceAmpersand(buf));
			UINT j = BuildMenuLevel(m->GetSubMenu(m->GetMenuItemCount() - 1), l + 1, i, finish, ExtractLevelPath(m_PlaylistManager.GetPlaylist(i), l + 1), counter);
			if (Current >= i && Current <= j)
				m->SetDefaultItem(m->GetMenuItemCount() - 1, TRUE);
			i = j;
			continue;
		}
		cnt++;
		buf.Format(_T("%d|%s"), i, m_PlaylistManager.GetFilename(i));
		m->AppendMenu(MF_STRING, IDM_TRACKS + i, buf);
		if (TPC && cnt % TPC == 0 && cnt > 0)
			m->ModifyMenu(IDM_TRACKS + i, MF_BYCOMMAND | MF_MENUBARBREAK | MF_STRING, IDM_TRACKS + i, buf);
		if (i == Current)
			m->SetDefaultItem(m->GetMenuItemCount() - 1, TRUE);
	}
	//On comble par des blancs
	if (TPC && cnt / TPC > 0)
		for (UINT i = cnt + 1; i % TPC != 0; i++)
			m->AppendMenu(MF_STRING, IDM_TRACKS - 1, _T(""));
	if (counter != NULL)
		*counter = cnt;
	return i - 1;
}

void CMainWnd::BuildPrevNextMenu(CMenu* m)
{
	UINT start, finish, j, oldCurrent;
	int i, cnt = -1;
	GetAlbumRange(&start, &finish);
	CString folder = m_PlaylistManager.GetPlaylist(Current);
	folder = m_PlaylistManager.GetPlaylist(Current).Left(m_PlaylistManager.GetPlaylist(Current).ReverseFind('\\'));
	if (folder.Find('\\', 0) != -1 && !ShuffleMode)
	{
		folder = folder.Right(folder.GetLength() - folder.ReverseFind('\\') - 1);
		m->AppendMenu(MF_STRING, 65535, folder);
		m->AppendMenu(MF_SEPARATOR);
	}
	oldCurrent = Current;
	if (ShuffleMode)
		for (i = 0; i < 3 && i < m_PlaylistManager.GetSize() / 2; i++)
			Current = MyRandom(false);
	else
		for (i = 0; i < 2 && i < m_PlaylistManager.GetSize() / 2; i++)
		{
			if (Current == start)
				Current = finish;
			else
				Current--;
		}
	for (i = 0; i < 5 && i < m_PlaylistManager.GetSize(); i++)
	{
		if (ShuffleMode)
			j = MyRandom(true);
		else
			j = Current;
		CString buf;
		buf.Format(_T("%d|%s"), j, m_PlaylistManager.GetPlaylist(j));
		cnt++;
		m->AppendMenu(MF_STRING, IDM_TRACKS + j, buf);
		if (TPC && (cnt + 1) % TPC == 0 && (cnt + 1) > 0 && (cnt + 1) < 5)
		{
			folder = m_PlaylistManager.GetPlaylist(j).Left(m_PlaylistManager.GetPlaylist(j).ReverseFind('\\'));
			if (folder.Find('\\', 0) != -1 && !ShuffleMode)
			{
				folder = folder.Right(folder.GetLength() - folder.ReverseFind('\\') - 1);
				m->AppendMenu(MF_MENUBARBREAK | MF_STRING, 65535, folder);
				m->AppendMenu(MF_SEPARATOR);
			}
		}

		if (i == 2)
			m->SetDefaultItem(m->GetMenuItemCount() - 1, TRUE);
		if (ShuffleMode)
			Current = j;
		else if (Current == finish)
			Current = start;
		else
			Current++;
	}
	//On comble par des blancs
	if (TPC && cnt / TPC > 0)
		for (UINT i = cnt + 1; i % TPC != 0; i++)
			m->AppendMenu(MF_STRING, IDM_TRACKS - 1, _T(""));
	Current = oldCurrent;
}

void CMainWnd::ClickAction(int action)
{
	TRACE(_T("Action %d\n"), action);
	switch (action)
	{
		// play-pause
	case 1:
		switch (::SendMessage(WAEmuWnd->m_hWnd, WM_WA_IPC, 0, IPC_ISPLAYING))
		{
		case PLAYER_MODE_CLOSE :
			::SendMessage(WAEmuWnd->m_hWnd, WM_COMMAND, WINAMP_BUTTON2, 0);
			break;
		case PLAYER_MODE_PAUSED :
		case PLAYER_MODE_PLAYING :
			::SendMessage(WAEmuWnd->m_hWnd, WM_COMMAND, WINAMP_BUTTON3, 0);
			break;
		}
		break;
		// stop-play
	case 2:
		switch (::SendMessage(WAEmuWnd->m_hWnd, WM_WA_IPC, 0, IPC_ISPLAYING))
		{
		case PLAYER_MODE_CLOSE :
			::SendMessage(WAEmuWnd->m_hWnd, WM_COMMAND, WINAMP_BUTTON2, 0);
			break;
		case PLAYER_MODE_PAUSED :
		case PLAYER_MODE_PLAYING :
			::SendMessage(WAEmuWnd->m_hWnd, WM_COMMAND, WINAMP_BUTTON4, 0);
			break;
		}
		break;
		// prev
	case 3:
		::SendMessage(WAEmuWnd->m_hWnd, WM_COMMAND, WINAMP_BUTTON1, 0);
		break;
		// next
	case 4:
		::SendMessage(WAEmuWnd->m_hWnd, WM_COMMAND, WINAMP_BUTTON5, 0);
		break;
		// tracks
	case 5:
		ShowTracksMenu();
		break;
		// minibar
	case 6:
		OnPopupMenuSeek();
		break;
		// playlist
	case 7:
		::SetForegroundWindow(m_hWnd);
		OnPopupMenuPlaylistEditor();
		break;
		// equalizer
	case 8:
		OnPopupMenuEqualizer();
		break;
		// mp3 info
	case 9:
		if (m_PlaylistManager.GetSize() > 0)
			OnPopupMenuFileInfo();
		break;
	case 10:
		ShowPopup();
		break;
		// album tracks
	case 11:
		ShowTracksMenu(0, true);
		break;
	}
}
void CMainWnd::DisplayTooltip()
{
	Shell_NotifyIcon(NIM_MODIFY, &nid);
	m_tray_pos.InitializePositionTracking(pMainFrame->m_hWnd, 1);
	CPoint ptIcon;
	BOOL bIconFound = m_tray_pos.GetTrayIconPosition(ptIcon);
	if (bIconFound == -1)
		return;

	m_PlaylistManager.ReadTagAT(Current, true);
	m_tray_pos.RestoreTrayIcon(PlayIcon);
	CString str;

	str = pMainFrame->toolTipText;
	str.Replace(_T("\\r"), _T("\r"));
	str.Replace(_T("\\n"), _T("\n"));
	str.Replace(_T("\\t"), _T("\t"));

	str.Replace(_T("%1"), m_PlaylistManager.GetArtist(Current));
	str.Replace(_T("%2"), m_PlaylistManager.GetTitle(Current));
	str.Replace(_T("%3"), m_PlaylistManager.GetAlbum(Current));
	str.Replace(_T("%4"), m_PlaylistManager.GetYear(Current));
	str.Replace(_T("%5"), m_PlaylistManager.GetComment(Current));
	str.Replace(_T("%6"), m_PlaylistManager.GetDuration(Current));
	str.Replace(_T("%7"), m_PlaylistManager.GetFilename(Current));


	//    str.Format(_T("<table><tr><td><icon idres=301></td><td><center><h2>%s</h2></center></td><td><right><a><icon idres=248 width=16 height=16 style=g hotstyle></a></right></td></tr><tr><td colspan=3><hr color=blue></td></tr><tr><td><bmp idres=138 mask style=d></td><td colspan=2>"),
	//  			 m_PlaylistManager.GetTitle(Current));
	CString str1;

	//we were able to find out icon on the screen
	str1.Format(_T("<table><tr><td>Fichier :</td><td> %s</td></tr><tr><td>Artiste :</td><td> %s</td></tr><tr><td>Album :</td><td> %s</td></tr><tr><td>Anne :</td><td> %s</td></tr><tr><td>Commentaire :</td><td> %s</td></tr><tr><td>Dure :</td><td> %s</td></tr></table>"), m_PlaylistManager.GetFilename(Current), m_PlaylistManager.GetArtist(Current), m_PlaylistManager.GetAlbum(Current), m_PlaylistManager.GetYear(Current), m_PlaylistManager.GetComment(Current), m_PlaylistManager.GetDuration(Current));
	//    str += str1;
	//    str += _T("</td></tr></table>");
	if (showToolTip)
	{
		SetTimer(HIDE_TOOLTIP, toolTipTime * 1000, NULL);
		m_traytip.ShowHelpTooltip(&ptIcon, str);
	}
}

void CMainWnd::DrawFragment(DRAWITEMSTRUCT* psI, int frg)
{
	//Rfrence rapport au haut du menu
	CMenuItem m((HMENU)psI->hwndItem, psI->itemID, false);
	HDC hDC = CreateCompatibleDC(psI->hDC);
	int oWidth = 90;
	int oHeight = 90 / 5;
	if (psI->itemState & ODS_SELECTED)
		ButtonsDn = (HBITMAP)SelectObject(hDC, ButtonsDn);
	else
		Buttons = (HBITMAP)SelectObject(hDC, Buttons);

	int imgWidth = (oWidth * (psI->rcItem.bottom - psI->rcItem.top) < oHeight * (psI->rcItem.right - psI->rcItem.left)) ? oWidth * (psI->rcItem.bottom - psI->rcItem.top) / oHeight : oWidth;
	int imgHeight = (oWidth * (psI->rcItem.bottom - psI->rcItem.top) < oHeight * (psI->rcItem.right - psI->rcItem.left)) ? oHeight* imgWidth / oWidth : oHeight;

	HDC hdc1 = CreateCompatibleDC(psI->hDC);
	HDC hdc2 = CreateCompatibleDC(psI->hDC);
	HDC hdc3 = CreateCompatibleDC(psI->hDC);
	HDC hdc4 = CreateCompatibleDC(psI->hDC);
	HDC hdc5 = CreateCompatibleDC(psI->hDC);
	//Masque binaire
	HBITMAP bm1 = CreateBitmap(imgWidth, imgHeight, 1, 1, NULL);
	HBITMAP bm2 = CreateBitmap(imgWidth, imgHeight, 1, 1, NULL);
	HBITMAP bm3 = CreateCompatibleBitmap(hDC, imgWidth, imgHeight);
	HBITMAP bm4 = CreateCompatibleBitmap(hDC, imgWidth, imgHeight);
	HBITMAP bm5 = CreateCompatibleBitmap(hDC, psI->rcItem.right - psI->rcItem.left, imgHeight);
	SelectObject(hdc1, bm1);
	SelectObject(hdc2, bm2);
	SelectObject(hdc3, bm3);
	SelectObject(hdc4, bm4);
	SelectObject(hdc5, bm5);

	//Image Original
	COLORREF cColor = SetBkColor(hDC, RGB(255, 0, 255));
	StretchBlt(hdc1, 0, 0, imgWidth, imgHeight, hDC, 0, oHeight * frg, oWidth, oHeight, SRCCOPY);
	//Image invers
	StretchBlt(hdc2, 0, 0, imgWidth, imgHeight, hDC, 0, oHeight * frg, oWidth, oHeight, NOTSRCCOPY);
	if (XPLookNFeel)
		SetBkColor(hDC, cColor);
	else
		SetBkColor(hdc1, cColor);
	if (!XPLookNFeel)
		CDC::FromHandle(hdc5)->FillSolidRect(0, 0, psI->rcItem.right - psI->rcItem.left, imgHeight, (COLORREF)((psI->itemState & ODS_SELECTED) != 0) ? GetSysColor(COLOR_HIGHLIGHT) : GetSysColor(COLOR_MENU));
	else
		m.Draw(CDC::FromHandle(hdc5), &CRect(0, 0, psI->rcItem.right - psI->rcItem.left, imgHeight), (psI->itemState & ODS_SELECTED) != 0, LOBYTE(LOWORD(psI->itemData)) == 0, (psI->itemState & ODS_HOTLIGHT) != 0, (psI->itemState & ODS_INACTIVE) != 0, (psI->itemState & ODS_NOACCEL) != 0);
	StretchBlt(hdc3, 0, 0, imgWidth, imgHeight, hdc5, (psI->rcItem.right - psI->rcItem.left - imgWidth) / 2, 0, imgWidth, imgHeight, SRCCOPY);
	StretchBlt(hdc4, 0, 0, imgWidth, imgHeight, hDC, 0, oHeight * frg, oWidth, oHeight, SRCCOPY);
	//Fond Orignal
	//Et entre Image invers et fond
	BitBlt(hdc3, 0, 0, imgWidth, imgHeight, hdc1, 0, 0, SRCAND);
	BitBlt(hdc4, 0, 0, imgWidth, imgHeight, hdc2, 0, 0, SRCAND);
	//Ou entre les 2 images obtenu
	BitBlt(hdc4, 0, 0, imgWidth, imgHeight, hdc3, 0, 0, SRCPAINT);
	BitBlt(hdc5, (psI->rcItem.right - psI->rcItem.left - imgWidth) / 2, 0, imgWidth, imgHeight, hdc4, 0, 0, SRCCOPY);
	BitBlt(psI->hDC, psI->rcItem.left, psI->rcItem.top, psI->rcItem.right - psI->rcItem.left, imgHeight, hdc5, 0, 0, SRCCOPY);

	DeleteDC(hdc1);DeleteDC(hdc2);DeleteDC(hdc3);DeleteDC(hdc4);DeleteDC(hdc5);
	DeleteObject(bm1);DeleteObject(bm2);DeleteObject(bm3);DeleteObject(bm4);DeleteObject(bm5);

	if (psI->itemState & ODS_SELECTED)
		ButtonsDn = (HBITMAP)SelectObject(hDC, ButtonsDn);
	else
		Buttons = (HBITMAP)SelectObject(hDC, Buttons);

	DeleteDC(hDC);
}

bool CMainWnd::ExtractLevel(CString s, int l, CString* r)
{
	int i, j;
	int start = 0;
	if (s.Left(2) == "\\\\")
		start = 2;
	else if (s.Left(7) == "http://")
	{
		start = 7; l--;
	}
	else if (s.Left(6) == "ftp://")
	{
		start = 6; l--;
	}
	if (l == 0)
	{
		*r = s.Left(start);
		return true;
	}
	j = start - 1;
	for (i = start; i < s.GetLength(); i++)
	{
		if (s[i] == '\\' || s[i] == '/')
		{
			l--;
			if (l == 0)
			{
				*r = s.Mid(j + 1, i - j - 1);
				return true;
			}
			j = i;
		}
	}
	return false;
}

CString CMainWnd::ExtractLevelPath(CString s, int l)
{
	int i;
	int start = 0;
	if (s.Left(2) == "\\\\")
		start = 2;
	else if (s.Left(7) == "http://")
	{
		start = 7; l--;
	}
	else if (s.Left(6) == "ftp://")
	{
		start = 6; l--;
	}
	if (l == 0)
		return s.Left(start);
	for (i = start; i < s.GetLength(); i++)
	{
		if (s[i] == '\\' || s[i] == '/')
		{
			l--;
			if (l == 0)
				return s.Left(i + 1);
		}
	}
	return "";
}

void CMainWnd::FindMP3Files(CString path, bool nodups)
{
	CFileFind finder;
	CString fp, fext;
	CString posExt = "" + stdIn->GetFileExtensions();
	BOOL bWorking = finder.FindFile(path + "*.*");
	while (bWorking)
	{
		bWorking = finder.FindNextFile();
		fp = finder.GetFilePath();
		fext = fp.Mid(fp.ReverseFind('.') + 1, 3);fext.MakeUpper();
		if (posExt.Find(fext) != -1)
		{
			if (finder.GetLength() > 0)
			{
				CString nTag = fp;
				if (nodups)
				{
					bool flag = false;
					for (int i = 0; i < m_PlaylistManager.GetSize(); i++)
					{
						CString nom = m_PlaylistManager.GetPlaylist(i);
						if (nom.CompareNoCase(fp) == 0)
						{
							flag = true; 
							break;
						}
					}
					if (flag)
						continue;
				}
				m_PlaylistManager.Add(nTag);
				continue;
			}
		}
		if (finder.IsDirectory() && !finder.IsDots())
			FindMP3Files(fp + "\\", nodups);
	}
}

void CMainWnd::Finish()
{
	CRect rect;
	Created = false;

	m_PlaylistManager.StopReadAll();
	m_PlaylistManager.StopPerformAutomation();

	if(PlaylistDlg != NULL) {
		PlaylistDlg->StopRemoveDuplicateEntries();
		PlaylistDlg->StopRemovene();
		PlaylistDlg->StopSort();
	}

	if (AutDlg != NULL)
		AutDlg->DestroyWindow();

	if (SettingsDialog != NULL)
		SettingsDialog->DestroyWindow();

	if (firstPass)
	{
		firstPass = false;
		if (SeekDialog != NULL && SeekDialog->m_hWnd != NULL)
		{
			SeekDialog->SaveWindowPosAndDestroy();
			theApp.WriteProfileInt(_T("Preferences"), _T("BarAtStartup"), 1);
		}
		else
			theApp.WriteProfileInt(_T("Preferences"), _T("BarAtStartup"), 0);
		if (MP3InfoDlg != NULL && MP3InfoDlg->m_hWnd != NULL)
			MP3InfoDlg->SaveWindowPosAndDestroy();

		if (PlaylistDlg != NULL && PlaylistDlg->m_hWnd != NULL)
		{
			//TODO A corriger
			//PlaylistDlg->SaveWindowPosAndDestroy();
			theApp.WriteProfileInt(_T("Preferences"), _T("PlaylistAtStartup"), 1);
		}
		else
			theApp.WriteProfileInt(_T("Preferences"), _T("PlaylistAtStartup"), 0);
		if (EqualizerDlg != NULL && EqualizerDlg->m_hWnd != NULL)
		{
			EqualizerDlg->SaveWindowPosAndDestroy();
			theApp.WriteProfileInt(_T("Preferences"), _T("EQAtStartup"), 1);
		}
		else
			theApp.WriteProfileInt(_T("Preferences"), _T("EQAtStartup"), 0);


		SaveThePlaylist();

		theApp.WriteProfileInt(_T("Preferences"), _T("XPLookNFeel"), XPLookNFeel ? 1 : 0);

		if (W2Fenabled)
			PerformW2File(true);

		SaveSettings();
	}
	if (::SendMessage(WAEmuWnd->m_hWnd, WM_WA_IPC, 0, IPC_ISPLAYING) != PLAYER_MODE_CLOSE)
		::SendMessage(WAEmuWnd->m_hWnd, WM_COMMAND, WINAMP_BUTTON4, 0);

	theApp.WriteProfileInt(_T("Preferences"), _T("CurrentInPlaylist"), Current);

	Shell_NotifyIcon(NIM_DELETE, &nid);

	DeleteObject(hMenuBitmap);
	DeleteObject(Buttons);
	DeleteObject(ButtonsDn);
	DeleteObject(PlayIcon);
	DeleteObject(StopIcon);
	DeleteObject(PauseIcon);
	DeleteObject(StpIcon);
	DeleteObject(StpIconSm);

	if (AboutDialog != NULL && AboutDialog->m_hWnd != NULL)
		AboutDialog->DestroyWindow();
	if (AboutDialog != NULL)
		delete AboutDialog;
	AboutDialog = NULL;

	if (SeekDialog != NULL && SeekDialog->m_hWnd != NULL)
		SeekDialog->DestroyWindow();
	if (SeekDialog != NULL)
		delete SeekDialog;
	SeekDialog = NULL;

	if (AutDlg != NULL && AutDlg->m_hWnd != NULL)
		AutDlg->DestroyWindow();
	if (AutDlg != NULL)
		delete AutDlg;
	AutDlg = NULL;

	if (SettingsDialog != NULL && SettingsDialog->m_hWnd != NULL)
		SettingsDialog->DestroyWindow();
	if (SettingsDialog != NULL)
		delete SettingsDialog;
	SettingsDialog = NULL;

	if (MP3InfoDlg != NULL && MP3InfoDlg->m_hWnd != NULL)
		MP3InfoDlg->DestroyWindow();
	if (MP3InfoDlg != NULL)
		delete MP3InfoDlg;
	MP3InfoDlg = NULL;

	if (PlaylistDlg != NULL && PlaylistDlg->m_hWnd != NULL)
		PlaylistDlg->SaveWindowPosAndDestroy();
	if (PlaylistDlg != NULL)
		delete PlaylistDlg;
	PlaylistDlg = NULL;

	if (EqualizerDlg != NULL && EqualizerDlg->m_hWnd != NULL)
		EqualizerDlg->SaveWindowPosAndDestroy();
	if (EqualizerDlg != NULL)
		delete EqualizerDlg;
	EqualizerDlg = NULL;

	delete pVolume;
	pVolume = NULL;
}

void CMainWnd::GetAlbumRange(UINT* b, UINT* e, bool force, int track)
{
	if (track == -1)
		track = Current;
	if (!AlbumStop && !force)
	{
		*b = 0;
		*e = m_PlaylistManager.GetSize() - 1;
	}
	else
	{
		CString lastfolder = m_PlaylistManager.GetPlaylist(track).Left(m_PlaylistManager.GetPlaylist(track).ReverseFind('\\'));
		UINT i;
		for (i = track; i > 0; i--)
		{
			if (lastfolder.CompareNoCase(m_PlaylistManager.GetPlaylist(i).Left(m_PlaylistManager.GetPlaylist(i).ReverseFind('\\'))) != 0)
				break;
		}
		*b = i + 1;
		if (i + 1 == m_PlaylistManager.GetSize() || Current == 0)
			*b = 0;
		for (i = track + 1; i < (UINT)m_PlaylistManager.GetSize(); i++)
		{
			if (lastfolder.CompareNoCase(m_PlaylistManager.GetPlaylist(i).Left(m_PlaylistManager.GetPlaylist(i).ReverseFind('\\'))) != 0)
				break;
		}
		*e = i - 1;
	}
}

int CMainWnd::GetMaxLevel(UINT start, UINT finish)
{
	int ml = 0, cl;
	UINT i;
	int j, st;
	CString buf;

	for (i = start; i < finish; i++)
	{
		cl = 0;
		if (m_PlaylistManager.GetPlaylist(i).Left(2) == "\\\\")
			st = 2;
		else if (m_PlaylistManager.GetPlaylist(i).Left(7) == "http://")
		{
			st = 7; cl++;
		}
		else if (m_PlaylistManager.GetPlaylist(i).Left(6) == "ftp://")
		{
			st = 6; cl++;
		}
		else
			st = 0;
		for (j = st; j < m_PlaylistManager.GetPlaylist(i).GetLength(); j++)
		{
			if (m_PlaylistManager.GetPlaylist(i)[j] == '\\' || m_PlaylistManager.GetPlaylist(i)[j] == '/')
				cl++;
		}
		if (cl > ml)
			ml = cl;
	}
	return ml;
}

void CMainWnd::GetPosForMenu(LPPOINT point)
{
	CRect rect1, rect2, rect3;
	::GetWindowRect(::FindWindow(_T("Shell_TrayWnd"), NULL), &rect1);
	::GetWindowRect(::GetDesktopWindow(), &rect2);
	::SystemParametersInfo(SPI_GETWORKAREA, 0, &rect3, 0);
	if (rect1.right < rect2.right)
	{
		point->x = rect3.left;
		point->y = rect3.top;
	}
	else if (rect1.left > rect2.left)
	{
		point->x = rect3.right;
		point->y = rect3.top;
	}
	else if (rect1.bottom < rect2.bottom)
	{
		point->x = rect3.right;
		point->y = rect3.top;
	}
	else
	{
		point->x = rect3.right;
		point->y = rect3.bottom;
	}
}
int CMainWnd::GetStartLevel(UINT start, UINT finish)
{
	int sl = 0;
	bool flag;
	UINT i;
	CString buf;

	do
	{
		sl++;
		buf = ExtractLevelPath(m_PlaylistManager.GetPlaylist(start), sl);
		if (buf == "")
		{
			flag = false; break;
		}
		flag = true;
		for (i = start; i < finish; i++)
		{
			if (m_PlaylistManager.GetPlaylist(i).Left(buf.GetLength()).CompareNoCase(buf) != 0)
			{
				flag = false;
				break;
			}
		}
	}
	while (flag);
	return sl - 1;
}
BOOL CMainWnd::PreTranslateMessage(MSG* pMsg)
{
	m_traytip.RelayEvent(pMsg);
	return CWnd::PreTranslateMessage(pMsg);
}

void CMainWnd::GoNext()
{
	if (GapBtwTracks > 0)
		Sleep(GapBtwTracks);

	if (m_RepeatTrack == 1)
	{
		if (::SendMessage(WAEmuWnd->m_hWnd, WM_WA_IPC, 0, IPC_ISPLAYING) != PLAYER_MODE_CLOSE)
			::SendMessage(WAEmuWnd->m_hWnd, WM_COMMAND, WINAMP_BUTTON4, 0);
		COPYDATASTRUCT copyData;
		copyData.dwData = IPC_PLAYFILE;
		copyData.cbData = m_PlaylistManager.GetPlaylist(Current).GetLength();

		copyData.lpData = (PVOID)(LPCTSTR)m_PlaylistManager.GetPlaylist(pMainFrame->Current);
		::SendMessage(WAEmuWnd->m_hWnd, WM_COPYDATA, 0, (WPARAM)&copyData);
	}
	else
		::SendMessage(WAEmuWnd->m_hWnd, WM_COMMAND, WINAMP_BUTTON5, 0);
	UpdMBTR();
	if (PlaylistDlg != NULL && PlaylistDlg->m_hWnd != NULL)
		PlaylistDlg->m_playlistat.RedrawWindow(NULL, NULL, RDW_INVALIDATE | RDW_UPDATENOW);
}

void CMainWnd::HideTooltip()
{
	m_traytip.HideTooltip();
}

LRESULT CMainWnd::IconNotify(WPARAM wParam, LPARAM lParam)
{
	if (lParam == WM_MOUSEMOVE)
	{
		GetCursorPos(&CurMousePos);
		if (!startToolTip)
		{
			startToolTip = true;
			SetTimer(DELAY_FROM_TOOLTIP, 1000, NULL);
		}
		return TRUE;
	}
	if (NoClicks == 1)
	{
		KillTimer(DELAY_FROM_TOOLTIP);
		SetTimer(DELAY_FROM_BEBORE_RESTART_TOOLTIP, 1000, NULL);
		if (lParam == WM_MOUSEMOVE || lParam == WM_RBUTTONUP || lParam == WM_LBUTTONUP)
			ShowPopup();
	}
	else
	{
		KillTimer(DELAY_FROM_TOOLTIP);
		SetTimer(DELAY_FROM_BEBORE_RESTART_TOOLTIP, 1000, NULL);
		switch (lParam)
		{
		case WM_LBUTTONDBLCLK:
		case WM_LBUTTONDOWN:
			lbd = true;
			if (lbs == 0 && rbs == 0 && PressAndHold < 2)
				SetTimer(LBUTTON_CLICK, GetDoubleClickTime(), NULL);
			return TRUE;
			break;
		case WM_RBUTTONDBLCLK:
		case WM_RBUTTONDOWN:
			rbd = true;
			if (lbs == 0 && rbs == 0 && PressAndHold < 2)
				SetTimer(RBUTTON_CLICK, GetDoubleClickTime(), NULL);
			return TRUE;
			break;
		case WM_RBUTTONUP:
			if (rbd || rbs > 0)
			{
				KillTimer(1);KillTimer(RBUTTON_UP);KillTimer(LBUTTON_CLICK);KillTimer(RBUTTON_CLICK);
				if (!TraySeeking)
				{
					if (rbs == 0 && RClicks2 == 0 && RClicks3 == 0 && RClicks4 == 0)
						ClickAction(RClicks1);
					else
					{
						rbs++;
						SetTimer(RBUTTON_UP, GetDoubleClickTime(), NULL);
					}
				}
				TraySeeking = false;
				rbd = false;
			}
			return TRUE;
			break;
		case WM_LBUTTONUP:
			if (lbd || lbs > 0)
			{
				KillTimer(LBUTTON_UP);KillTimer(RBUTTON_UP);KillTimer(LBUTTON_CLICK);KillTimer(RBUTTON_CLICK);
				if (!TraySeeking)
				{
					if (lbs == 0 && Clicks2 == 0 && Clicks3 == 0 && Clicks4 == 0)
						ClickAction(Clicks1);
					else
					{
						lbs++;
						SetTimer(LBUTTON_UP, GetDoubleClickTime(), NULL);
					}
				}
				TraySeeking = false;
				lbd = false;
			}
			return TRUE;
			break;
		}
	}
	TRACE(_T("Message non reu %d %d\n"), wParam, lParam);
	return FALSE;
}
void CMainWnd::LoadSkin()
{
	CString fname = CurrentSkin;
	if (fname == "")
	{
		Buttons = (HBITMAP)::LoadImage(theApp.m_hInstance, MAKEINTRESOURCE(IDB_BUTTONS), IMAGE_BITMAP, 90, 90, LR_DEFAULTCOLOR);
		ButtonsDn = (HBITMAP)::LoadImage(theApp.m_hInstance, MAKEINTRESOURCE(IDB_BUTTONSDN), IMAGE_BITMAP, 90, 90, LR_DEFAULTCOLOR);
		PlayIcon = (HICON)::LoadImage(theApp.m_hInstance, MAKEINTRESOURCE(IDI_PLAY), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR);
		StopIcon = (HICON)::LoadImage(theApp.m_hInstance, MAKEINTRESOURCE(IDI_STOP), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR);
		PauseIcon = (HICON)::LoadImage(theApp.m_hInstance, MAKEINTRESOURCE(IDI_PAUSE), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR);
		PrevIcon = (HICON)::LoadImage(theApp.m_hInstance, MAKEINTRESOURCE(IDI_PREV), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR);
		NextIcon = (HICON)::LoadImage(theApp.m_hInstance, MAKEINTRESOURCE(IDI_NEXT), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR);
		return;
	}
	CStdioFile sknfile;
	if (!sknfile.Open(fname, CFile::modeRead | CFile::shareDenyNone))
	{
		Buttons = (HBITMAP)::LoadImage(theApp.m_hInstance, MAKEINTRESOURCE(IDB_BUTTONS), IMAGE_BITMAP, 90, 90, LR_DEFAULTCOLOR);
		ButtonsDn = (HBITMAP)::LoadImage(theApp.m_hInstance, MAKEINTRESOURCE(IDB_BUTTONSDN), IMAGE_BITMAP, 90, 90, LR_DEFAULTCOLOR);
		PlayIcon = (HICON)::LoadImage(theApp.m_hInstance, MAKEINTRESOURCE(IDI_PLAY), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR);
		StopIcon = (HICON)::LoadImage(theApp.m_hInstance, MAKEINTRESOURCE(IDI_STOP), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR);
		PauseIcon = (HICON)::LoadImage(theApp.m_hInstance, MAKEINTRESOURCE(IDI_PAUSE), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR);
		PrevIcon = (HICON)::LoadImage(theApp.m_hInstance, MAKEINTRESOURCE(IDI_PREV), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR);
		NextIcon = (HICON)::LoadImage(theApp.m_hInstance, MAKEINTRESOURCE(IDI_NEXT), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR);
		CurrentSkin = "";
		return;
	}
	CString buf;
	CString CurPath = fname.Left(fname.ReverseFind('\\') + 1);
	if (sknfile.ReadString(buf))
	{
		if (buf != "*")
		{
			if ((Buttons = (HBITMAP)::LoadImage(0, CurPath + buf, IMAGE_BITMAP, 90, 90, LR_LOADFROMFILE | LR_DEFAULTCOLOR)) == NULL)
				Buttons = (HBITMAP)::LoadImage(theApp.m_hInstance, MAKEINTRESOURCE(IDB_BUTTONS), IMAGE_BITMAP, 90, 90, LR_DEFAULTCOLOR);
		}
		else
			Buttons = (HBITMAP)::LoadImage(theApp.m_hInstance, MAKEINTRESOURCE(IDB_BUTTONS), IMAGE_BITMAP, 90, 90, LR_DEFAULTCOLOR);
	}
	else
		Buttons = (HBITMAP)::LoadImage(theApp.m_hInstance, MAKEINTRESOURCE(IDB_BUTTONS), IMAGE_BITMAP, 90, 90, LR_DEFAULTCOLOR);

	if (sknfile.ReadString(buf))
	{
		if (buf != "*")
		{
			if ((ButtonsDn = (HBITMAP)::LoadImage(0, CurPath + buf, IMAGE_BITMAP, 90, 90, LR_LOADFROMFILE | LR_DEFAULTCOLOR)) == NULL)
				ButtonsDn = (HBITMAP)::LoadImage(theApp.m_hInstance, MAKEINTRESOURCE(IDB_BUTTONSDN), IMAGE_BITMAP, 90, 90, LR_DEFAULTCOLOR);
		}
		else
			ButtonsDn = (HBITMAP)::LoadImage(theApp.m_hInstance, MAKEINTRESOURCE(IDB_BUTTONSDN), IMAGE_BITMAP, 90, 90, LR_DEFAULTCOLOR);
	}
	else
		ButtonsDn = (HBITMAP)::LoadImage(theApp.m_hInstance, MAKEINTRESOURCE(IDB_BUTTONSDN), IMAGE_BITMAP, 90, 90, LR_DEFAULTCOLOR);
	if (sknfile.ReadString(buf))
	{
		if (buf != "*")
		{
			if ((PlayIcon = (HICON)::LoadImage(0, CurPath + buf, IMAGE_ICON, 16, 16, LR_LOADFROMFILE)) == NULL)
				PlayIcon = (HICON)::LoadImage(theApp.m_hInstance, MAKEINTRESOURCE(IDI_PLAY), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR);
		}
		else
			PlayIcon = (HICON)::LoadImage(theApp.m_hInstance, MAKEINTRESOURCE(IDI_PLAY), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR);
	}
	else
		PlayIcon = (HICON)::LoadImage(theApp.m_hInstance, MAKEINTRESOURCE(IDI_PLAY), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR);
	if (sknfile.ReadString(buf))
	{
		if (buf != "*")
		{
			if ((PauseIcon = (HICON)::LoadImage(0, CurPath + buf, IMAGE_ICON, 16, 16, LR_LOADFROMFILE)) == NULL)
				PauseIcon = (HICON)::LoadImage(theApp.m_hInstance, MAKEINTRESOURCE(IDI_PAUSE), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR);
		}
		else
			PauseIcon = (HICON)::LoadImage(theApp.m_hInstance, MAKEINTRESOURCE(IDI_PAUSE), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR);
	}
	else
		PauseIcon = (HICON)::LoadImage(theApp.m_hInstance, MAKEINTRESOURCE(IDI_PAUSE), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR);
	if (sknfile.ReadString(buf))
	{
		if (buf != "*")
		{
			if ((StopIcon = (HICON)::LoadImage(0, CurPath + buf, IMAGE_ICON, 16, 16, LR_LOADFROMFILE)) == NULL)
				StopIcon = (HICON)::LoadImage(theApp.m_hInstance, MAKEINTRESOURCE(IDI_STOP), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR);
		}
		else
			StopIcon = (HICON)::LoadImage(theApp.m_hInstance, MAKEINTRESOURCE(IDI_STOP), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR);
	}
	else
		StopIcon = (HICON)::LoadImage(theApp.m_hInstance, MAKEINTRESOURCE(IDI_STOP), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR);
	if (sknfile.ReadString(buf))
	{
		if (buf != "*")
		{
			if ((PrevIcon = (HICON)::LoadImage(0, CurPath + buf, IMAGE_ICON, 16, 16, LR_LOADFROMFILE)) == NULL)
				PrevIcon = (HICON)::LoadImage(theApp.m_hInstance, MAKEINTRESOURCE(IDI_PREV), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR);
		}
		else
			PrevIcon = (HICON)::LoadImage(theApp.m_hInstance, MAKEINTRESOURCE(IDI_PREV), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR);
	}
	else
		PrevIcon = (HICON)::LoadImage(theApp.m_hInstance, MAKEINTRESOURCE(IDI_PREV), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR);
	if (sknfile.ReadString(buf))
	{
		if (buf != "*")
		{
			if ((NextIcon = (HICON)::LoadImage(0, CurPath + buf, IMAGE_ICON, 16, 16, LR_LOADFROMFILE)) == NULL)
				NextIcon = (HICON)::LoadImage(theApp.m_hInstance, MAKEINTRESOURCE(IDI_NEXT), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR);
		}
		else
			NextIcon = (HICON)::LoadImage(theApp.m_hInstance, MAKEINTRESOURCE(IDI_NEXT), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR);
	}
	else
		NextIcon = (HICON)::LoadImage(theApp.m_hInstance, MAKEINTRESOURCE(IDI_NEXT), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR);

	m_imgList.Add(PlayIcon);
	m_imgList.Add(PauseIcon);
	m_imgList.Add(StopIcon);
	m_imgList.Add(PrevIcon);
	m_imgList.Add(NextIcon);

	sknfile.Close();
}

void CMainWnd::LoadThePlaylist(bool ForceStart)
{
	TCHAR buf2[MAX_PATH + 1];
	GetModuleFileName(theApp.m_hInstance, buf2, MAX_PATH);
	for (int i = _tcslen(buf2) - 1; i > 0; i--)
	{
		if (buf2[i] == '.')
		{
			buf2[i] = '\0';break;
		}
	}
	CString fname = buf2;
	fname += ".m3u";

	UINT FixedFirst = theApp.GetProfileInt(_T("Preferences"), _T("CurrentInPlaylist"), 0);
	Current = FixedFirst;
	OpenPlaylistEngine(fname, true, false);
	if (m_PlaylistManager.GetSize() == 0)
	{
		UpdMBTR();
		return;
	}
	if (RandomFirst || ShuffleMode)
		FirstInMyRandom = Current = MyRandom(true, true);
	else
		Current = FixedFirst < (UINT)m_PlaylistManager.GetSize() ? FixedFirst : 0;


	if (PlaylistDlg != NULL && PlaylistDlg->m_hWnd != NULL)
	{
		PlaylistDlg->FillPlaylist();
		PlaylistDlg->ShowFileName();
	}

	if (::SendMessage(WAEmuWnd->m_hWnd, WM_WA_IPC, 0, IPC_ISPLAYING) != PLAYER_MODE_CLOSE)
		::SendMessage(WAEmuWnd->m_hWnd, WM_COMMAND, WINAMP_BUTTON4, 0);

	bool first = true;
	int q = 0;
	do
	{
		if (!first)
		{
			if (Current < (UINT)m_PlaylistManager.GetSize() - 1)
				Current++;
			else
			{
				Current = 0;
				q++;
				if (m_Repeat != 1)
					break;
			}
		}
		else
			first = false;
		if (AutoPlay || ForceStart)
		{
			enqueueFileWithMetaStruct eFile;
			eFile.filename = (TCHAR *)(LPCTSTR)m_PlaylistManager.GetPlaylist(pMainFrame->Current);
			::SendMessage(WAEmuWnd->m_hWnd, IPC_PLAYFILE, 0, (WPARAM)&eFile);
		}
		else if (::SendMessage(WAEmuWnd->m_hWnd, WM_WA_IPC, 0, IPC_ISPLAYING) != PLAYER_MODE_CLOSE)
			::SendMessage(WAEmuWnd->m_hWnd, WM_COMMAND, WINAMP_BUTTON4, 0);
		break;
	}
	while (q < 2);

	UpdMBTR();
}

UINT CMainWnd::MyRandom(bool forw, bool init)
{
	if (init)
	{
		srand((unsigned)time(NULL));
		Current = rand() % m_PlaylistManager.GetSize();
	}
	else if (FirstInMyRandom == -1)
		FirstInMyRandom = Current;
	while (*rndpntr != Current)
	{
		if (rndpntr == rndlst + 49999)
			rndpntr = rndlst;
		else
			rndpntr++;
	}
	UINT b, e; GetAlbumRange(&b, &e);
	do
	{
		if (forw)
		{
			if (rndpntr == rndlst + 49999)
				rndpntr = rndlst;
			else
				rndpntr++;
		}
		else
		{
			if (rndpntr == rndlst)
				rndpntr = rndlst + 49999;
			else
				rndpntr--;
		}
	}
	while (*rndpntr < b || *rndpntr > e);
	return *rndpntr;
}

void CMainWnd::OnAutomationPerformNow()
{
	m_PlaylistManager.PerformAutomation();
	if (AutSaveset)
		SaveSettings();
}

void CMainWnd::OnAutomationSetup()
{
	AutDlg = AutDlg->getInstance(IDD_AUTOMATION, pMainFrame);
}

void CMainWnd::OnClose()
{
	Finish();    
	UnhookWindowsHookEx(hHook);
	CFrameWnd::OnClose();
}

BOOL CMainWnd::OnCopyData(CWnd* pWnd, COPYDATASTRUCT* pData)
{
	TRACE(_T("OnCopyData "));
	if (Created && pData->cbData == sizeof(MsgCmd))
	{
		MsgCmd cmd;
		memcpy(&cmd, pData->lpData, sizeof(MsgCmd));

		KillTimer(DELAY_FROM_EXP);

		CFileFind finder;
		bool Add = false;
		switch (cmd.command)
		{
		case 1:
			Add = true;break;
		}
		CString buf = cmd.fname;
		if (buf.GetLength() == 3 && buf[1] == ':')
			OpenFolder(buf.Left(2) + '\\', RandomFirst == 1, Add || AddFromExp, !(Add || AddFromExp));
		else if (finder.FindFile(buf))
		{
			finder.FindNextFile();
			if (finder.IsDirectory())
				OpenFolder(buf, RandomFirst == 1, Add || AddFromExp, !(Add || AddFromExp));
			else
			{
				if ((buf.Mid(buf.ReverseFind('.') + 1, 3)).CompareNoCase(_T("M3U")) == 0 || (buf.Mid(buf.ReverseFind('.') + 1, 3)).CompareNoCase(_T("PLS")) == 0)
					OpenPlaylist(buf, RandomFirst == 1, Add || AddFromExp, !(Add || AddFromExp));
				else
					OpenFile(buf, Add || AddFromExp, !(Add || AddFromExp));
			}
			AddRecent(buf);
			AddFromExp = true;
			SetTimer(DELAY_FROM_EXP, (DWORD)2000, NULL);
		}
		return TRUE;
	}
	else
		return FALSE;
}

int CMainWnd::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;
	hHook = SetWindowsHookEx(WH_CALLWNDPROC, CMainWnd::HookCallWndProc, 0, ::GetCurrentThreadId());
	m_traytip.Create(this);
	m_traytip.SetSize(CPPToolTip::PPTTSZ_MARGIN_CX, 4);
	m_traytip.SetSize(CPPToolTip::PPTTSZ_MARGIN_CY, 4);

	Start();

	mainMenu->LoadMenu(IDR_MENU);
	this->SetMenu(mainMenu); 

	CMenuXP::SetXPLookNFeel(this, XPLookNFeel);

	return 0;
}

void CMainWnd::OnDrawItem(int nIDCtl, LPDRAWITEMSTRUCT lpDrawItemStruct)
{
	if (lpDrawItemStruct->itemID == ID_POPUPMENU_PIC && lpDrawItemStruct->itemAction == ODA_DRAWENTIRE)
	{
		RECT sRect;
		COLORREF crGradientStart = startColor;
		COLORREF crGradientEnd = endColor;

		::GetMenuItemRect(m_hWnd, (HMENU)lpDrawItemStruct->hwndItem, 1, &sRect);
		int menutop = sRect.top;

		::GetMenuItemRect(m_hWnd, (HMENU)lpDrawItemStruct->hwndItem, ::GetMenuItemCount((HMENU)lpDrawItemStruct->hwndItem) - 1, &sRect);
		int menubottom = sRect.bottom;

		HDC hDC = CreateCompatibleDC(lpDrawItemStruct->hDC);
		hMenuBitmap = (HBITMAP)SelectObject(hDC, hMenuBitmap);
		lpDrawItemStruct->rcItem.top = 0;
		lpDrawItemStruct->rcItem.bottom = (menubottom - menutop);

		CRect rect2 = lpDrawItemStruct->rcItem;
		// Initialize fonts:
		LOGFONT lf;
		memset(&lf, 0, sizeof(LOGFONT));

		// Text will be rotated counter-clockwise 90 degrees.
		lf.lfOrientation = lf.lfEscapement = 900;

		// We need a TrueType font to get rotated text.  MS Sans //Serif won't cut it!
		lstrcpy(lf.lfFaceName, _T("Times"));

		lf.lfHeight = 20;
		lf.lfWeight = FW_DEMIBOLD;
		lf.lfItalic = TRUE;

		// Create a font with the tweaked attributes.
		CFont font;
		font.CreateFontIndirect(&lf);

		CFont* oldFond = CDC::FromHandle(lpDrawItemStruct->hDC)->SelectObject(&font);
		CMenuItem m((HMENU)lpDrawItemStruct->hwndItem, 0, true);

		HBITMAP bm = CreateCompatibleBitmap(hDC, lpDrawItemStruct->rcItem.right, lpDrawItemStruct->rcItem.bottom);

		SelectObject(hDC, bm);
		m.DrawGradient(CDC::FromHandle(hDC), XPLookNFeel ? crGradientEnd : RGB(0xFF, 0xFF, 0xFF), XPLookNFeel ? crGradientStart : RGB(0x00, 0x00, 0x00), &CRect(0, 0, lpDrawItemStruct->rcItem.right - lpDrawItemStruct->rcItem.left, lpDrawItemStruct->rcItem.bottom - lpDrawItemStruct->rcItem.top), true);
		lpDrawItemStruct->rcItem.right += 2;
		StretchBlt(lpDrawItemStruct->hDC, 0, 0, lpDrawItemStruct->rcItem.right, lpDrawItemStruct->rcItem.bottom, hDC, 0, 0, lpDrawItemStruct->rcItem.right, lpDrawItemStruct->rcItem.bottom, SRCCOPY);
		COLORREF oldColor = CDC::FromHandle(lpDrawItemStruct->hDC)->SetTextColor(XPLookNFeel ? crGradientEnd : RGB(0xFF, 0xFF, 0xFF));
		int oldBkMode = CDC::FromHandle(lpDrawItemStruct->hDC)->SetBkMode(TRANSPARENT);
		rect2.left -= 2;
		rect2.bottom += 15;
		CDC::FromHandle(lpDrawItemStruct->hDC)->DrawText("STP   2005", &rect2, DT_BOTTOM | DT_LEFT | DT_SINGLELINE);
		rect2.bottom -= 1;
		CDC::FromHandle(lpDrawItemStruct->hDC)->DrawText("STP   2005", &rect2, DT_BOTTOM | DT_LEFT | DT_SINGLELINE);

		CDC::FromHandle(lpDrawItemStruct->hDC)->SelectObject(oldFond);
		CDC::FromHandle(lpDrawItemStruct->hDC)->SetTextColor(oldColor);
		CDC::FromHandle(lpDrawItemStruct->hDC)->SetBkMode(oldBkMode);

		// clean up
		font.DeleteObject();

		hMenuBitmap = (HBITMAP)SelectObject(hDC, hMenuBitmap);
		DeleteDC(hDC);
	}
	if (!XPLookNFeel)
		CFrameWnd::OnDrawItem(nIDCtl, lpDrawItemStruct);
	//On affiche pas en style XP si image de gradient
	else if (lpDrawItemStruct->itemID == ID_POPUPMENU_PIC)
		CFrameWnd::OnDrawItem(nIDCtl, lpDrawItemStruct);
	else if (!CMenuXP::OnDrawItem(lpDrawItemStruct, m_hWnd))
		CFrameWnd::OnDrawItem(nIDCtl, lpDrawItemStruct);
	if (this->GraphicalButtons)
	{
		if (lpDrawItemStruct->itemID == ID_POPUPMENU_PLAY)
			DrawFragment(lpDrawItemStruct, 0);
		if (lpDrawItemStruct->itemID == ID_POPUPMENU_PAUSE)
			DrawFragment(lpDrawItemStruct, 1);
		if (lpDrawItemStruct->itemID == ID_POPUPMENU_STOP)
			DrawFragment(lpDrawItemStruct, 2);
		if (lpDrawItemStruct->itemID == ID_POPUPMENU_PREVIOUS)
			DrawFragment(lpDrawItemStruct, 3);
		if (lpDrawItemStruct->itemID == ID_POPUPMENU_NEXT)
			DrawFragment(lpDrawItemStruct, 4);
	}
}

void CMainWnd::OnDrivesMenu(UINT nID)
{
	CString CDLetter;
	CDLetter.Format(_T("%c"), (char)(nID - IDM_DRIVES) + 'C');
	OpenFolder(CDLetter + ":\\", RandomFirst == 1);
}

void CMainWnd::OnEndSession(BOOL bEnding)
{
	CFrameWnd::OnEndSession(bEnding);

	if (bEnding)
		Finish();
}

void CMainWnd::OnInitMenuPopup(CMenu* pMenu, UINT wParam, BOOL isActive)
{
	for (UINT i = 0; i < pMenu->GetMenuItemCount(); i++)
	{
		UINT menuID = pMenu->GetMenuItemID(i);
		CString buf, tmp;
		//Si on est pas sur un sous menu
		if (menuID != 0 && menuID != 0xFFFFFFFF)
		{
			pMenu->GetMenuString(i, tmp, MF_BYPOSITION);
			if (tmp.Find(_T("|")) != -1)
			{
				CString sNumber = tmp.Left(tmp.Find(_T("|")));
				int number = 0;
				for (int j = 0; j < sNumber.GetLength(); j++)
					number = number * 10 + sNumber.GetAt(j) - '0';
				buf = tmp.Mid(tmp.ReverseFind('\\') + 1, tmp.GetLength() - tmp.ReverseFind('\\') - 5);
				if (m_PlaylistManager.GetTag(number).GetLength() == 0)
					m_PlaylistManager.ReadTagAT(number);
				pMenu->ModifyMenu(menuID, MF_BYCOMMAND | pMenu->GetMenuState(menuID, MF_BYCOMMAND), menuID, m_PlaylistManager.GetTag(number));
			}
		}
	}
	CFrameWnd::OnInitMenuPopup(pMenu, wParam, isActive);
	CMenuXP::SetXPLookNFeel(this, pMenu->m_hMenu, XPLookNFeel);
}

void CMainWnd::OnMeasureItem(int nIDCtl, LPMEASUREITEMSTRUCT lpMeasureItemStruct)
{
	if (!XPLookNFeel)
	{
		if (lpMeasureItemStruct->itemID == ID_POPUPMENU_PLAY || lpMeasureItemStruct->itemID == ID_POPUPMENU_PAUSE || lpMeasureItemStruct->itemID == ID_POPUPMENU_STOP || lpMeasureItemStruct->itemID == ID_POPUPMENU_PREVIOUS || lpMeasureItemStruct->itemID == ID_POPUPMENU_NEXT)
		{
			lpMeasureItemStruct->itemHeight = 18;
			lpMeasureItemStruct->itemWidth = 90 - GetSystemMetrics(SM_CXMENUCHECK) * 2;
			//TODO A corriger
			//if(psI->itemWidth+2*GetSystemMetrics(SM_CXMENUCHECK)<90) psI->itemWidth=90-GetSystemMetrics(SM_CXMENUCHECK)*2;
		}
		else if (lpMeasureItemStruct->itemID == ID_POPUPMENU_PIC)
			lpMeasureItemStruct->itemWidth = 20 - GetSystemMetrics(SM_CXMENUCHECK);
	}
	else if (lpMeasureItemStruct->itemID == ID_POPUPMENU_PIC)
		lpMeasureItemStruct->itemWidth = 20 - GetSystemMetrics(SM_CXMENUCHECK);
	else
		CMenuXP::OnMeasureItem(lpMeasureItemStruct);
}

LRESULT CMainWnd::OnMenuChar(UINT nChar, UINT nFlags, CMenu* pMenu)
{
	if (XPLookNFeel && CMenuXP::IsOwnerDrawn(pMenu->m_hMenu))
		return CMenuXP::OnMenuChar(pMenu->m_hMenu, nChar, nFlags);  									 
	return CFrameWnd::OnMenuChar(nChar, nFlags, pMenu);
}

void CMainWnd::OnPopupMenuAbout()
{
	::SendMessage(WAEmuWnd->m_hWnd, WM_COMMAND, WINAMP_HELP_ABOUT, 0);
}

void CMainWnd::OnPopupMenuArtistTitle()
{
	ShowingMode = 1;
	m_PlaylistManager.SetShowingMode(ShowingMode);
	UpdMBTR();
}

void CMainWnd::OnPopupMenuDockToActiveWindow()
{
	if (DockSD)
		DockSD = false;
	else
		DockSD = true;
}

void CMainWnd::OnPopupMenuElapsedTime()
{
	::SendMessage(WAEmuWnd->m_hWnd, WM_COMMAND, WINAMP_OPTIONS_ELAPSED, 0);
}

void CMainWnd::OnPopupMenuEqualizer()
{
	::SendMessage(WAEmuWnd->m_hWnd, WM_COMMAND, WINAMP_OPTIONS_EQ, 0);
}
void CMainWnd::OnPopupMenuExit()
{
	::SendMessage(WAEmuWnd->m_hWnd, WM_COMMAND, WINAMP_BUTTON4, 0);
	::SendMessage(WAEmuWnd->m_hWnd, WM_COMMAND, WINAMP_FILE_QUIT, 0);
}

void CMainWnd::OnPopupMenuFileInfo()
{
	if (MP3InfoDlg == NULL || MP3InfoDlg != NULL && MP3InfoDlg->m_hWnd == NULL)
		MP3InfoDlg = MP3InfoDlg->getInstance(IDD_DIALOG_MP3INFO, pMainFrame);
	else
		MP3InfoDlg->SaveWindowPosAndDestroy();
}

void CMainWnd::OnPopupMenuFilename()
{
	ShowingMode = 2;
	m_PlaylistManager.SetShowingMode(ShowingMode);
	UpdMBTR();
}

void CMainWnd::OnPopupMenuNext()
{
	::SendMessage(WAEmuWnd->m_hWnd, WM_COMMAND, WINAMP_BUTTON5, 0);
}

void CMainWnd::OnPopupMenuOpenFile()
{
	::SendMessage(WAEmuWnd->m_hWnd, WM_COMMAND, WINAMP_FILE_PLAY, 0);
}

void CMainWnd::OnPopupMenuAddFile()
{
	UINT PLLPrev = m_PlaylistManager.GetSize();
	// open a file
	CFileDialog dialog(TRUE, _T("mp3"), _T(""), OFN_ALLOWMULTISELECT | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY, m_Helper.MyLoadString(IDS_OPENFILE), this);

	TCHAR buffer[_MAX_PATH*500 + 1];
	buffer[0] = '\0';
	dialog.m_ofn.lpstrFile = buffer;
	dialog.m_ofn.nMaxFile = _MAX_PATH * 500;
	dialog.m_ofn.lpstrInitialDir = LPCTSTR(pMainFrame->InitDir);
	int ret;
	ret = dialog.DoModal();
	if (ret == IDOK)
	{
		// a file was selected
		POSITION p = dialog.GetStartPosition();
		CString buf;
		while (p != NULL)
		{
			m_PlaylistManager.Add(dialog.GetNextPathName(p));
			pMainFrame->InitDir = buf.Left(buf.ReverseFind('\\') + 1);
		}
		if (PLLPrev == 0)
		{
			if (::SendMessage(WAEmuWnd->m_hWnd, WM_WA_IPC, 0, IPC_ISPLAYING) != PLAYER_MODE_CLOSE)
				::SendMessage(WAEmuWnd->m_hWnd, WM_COMMAND, WINAMP_BUTTON4, 0);
			if (pMainFrame->ShuffleMode)
				pMainFrame->FirstInMyRandom = pMainFrame->Current = pMainFrame->MyRandom(true, true);
			else
				pMainFrame->Current = 0;pMainFrame->FirstInMyRandom = 0xFFFFFFFF;

			bool first = true;
			int q = 0;
			do
			{
				if (!first)
				{
					if (pMainFrame->Current < (UINT)m_PlaylistManager.GetSize() - 1)
						pMainFrame->Current++;
					else
					{
						pMainFrame->Current = 0;
						q++;
						if (pMainFrame->m_Repeat != 1)
							break;
					}
				}
				else
					first = false;
				if (pMainFrame->AutoPlay)
				{
					enqueueFileWithMetaStruct eFile;
					eFile.filename = (TCHAR *)(LPCTSTR)m_PlaylistManager.GetPlaylist(pMainFrame->Current);
					::SendMessage(WAEmuWnd->m_hWnd, IPC_PLAYFILE, 0, (WPARAM)&eFile);
				}
				else if (::SendMessage(WAEmuWnd->m_hWnd, WM_WA_IPC, 0, IPC_ISPLAYING) != PLAYER_MODE_CLOSE)
					::SendMessage(WAEmuWnd->m_hWnd, WM_COMMAND, WINAMP_BUTTON4, 0);
				break;
			}
			while (q < 2);
		}
		pMainFrame->UpdMBTR();
	}
}

void CMainWnd::OnPopupMenuOpenFolder()
{
	if (opening)
		return;
	opening = true;

	LPMALLOC pMalloc;
	if (SHGetMalloc(&pMalloc) != NOERROR)
		return;

	BROWSEINFO bi;
	TCHAR buf[MAX_PATH];
	CString title;
	title = m_Helper.MyLoadString(IDS_OPENFOLDER);
	bi.hwndOwner = m_hWnd;
	bi.pszDisplayName = buf;
	bi.pidlRoot = NULL;
	bi.lpszTitle = (LPCTSTR)title;
	bi.ulFlags = BIF_RETURNONLYFSDIRS;
	bi.lpfn = BrowseCallbackProc;

	LPITEMIDLIST pidl;
	if ((pidl = SHBrowseForFolder(&bi)) == NULL)
	{
		pMalloc->Free(pidl);
		pMalloc ->Release();
		opening = false;
		return;
	}
	if (SHGetPathFromIDList(pidl, buf) == false)
	{
		pMalloc->Free(pidl);
		pMalloc ->Release();
		opening = false;
		return;
	}
	pMalloc->Free(pidl);
	pMalloc->Release();

	opening = false;

	AddRecent(buf);
	InitDir = buf;
	OpenFolder(buf, RandomFirst == 1);
}

void CMainWnd::OnPopupMenuAddFolder()
{
	if (opening)
		return;
	opening = true;

	LPMALLOC pMalloc;
	if (SHGetMalloc(&pMalloc) != NOERROR)
		return;

	BROWSEINFO bi;
	TCHAR buf[MAX_PATH];
	CString title;
	title = m_Helper.MyLoadString(IDS_OPENFOLDER);
	bi.hwndOwner = m_hWnd;
	bi.pszDisplayName = buf;
	bi.pidlRoot = NULL;
	bi.lpszTitle = (LPCTSTR)title;
	bi.ulFlags = BIF_RETURNONLYFSDIRS;
	bi.lpfn = BrowseCallbackProc;

	LPITEMIDLIST pidl;
	if ((pidl = SHBrowseForFolder(&bi)) == NULL)
	{
		pMalloc->Free(pidl);
		pMalloc ->Release();
		opening = false;
		return;
	}
	if (SHGetPathFromIDList(pidl, buf) == false)
	{
		pMalloc->Free(pidl);
		pMalloc ->Release();
		opening = false;
		return;
	}
	pMalloc->Free(pidl);
	pMalloc->Release();

	opening = false;

	AddRecent(buf);
	InitDir = buf;
	OpenFolder(buf, RandomFirst == 1, true);
}

void CMainWnd::OnPopupMenuOpenRecent(UINT nID)
{
	OpenRecent(nID - IDM_RECENT_FILES);
}

void CMainWnd::OnPopupMenuOpenUrl()
{
	if (opening)
		return;
	opening = true;

	CUrlDlg dialog(IDD_ENTER_URL_DIALOG);

	int ret;
	ret = dialog.DoModal();
	if (ret == IDOK)
	{
		CString buf = dialog.m_URL;
		buf.Replace(_T(" "), _T("%20"));
		OpenUrl(buf);
		AddRecent(buf);
	}
	opening = false;
}

void CMainWnd::OnPopupMenuPause()
{
	::SendMessage(WAEmuWnd->m_hWnd, WM_COMMAND, WINAMP_BUTTON3, 0);
}

void CMainWnd::OnPopupMenuPlay()
{
	::SendMessage(WAEmuWnd->m_hWnd, WM_COMMAND, WINAMP_BUTTON2, 0);
}

void CMainWnd::OnPopupMenuPlaylistEditor()
{
	::SendMessage(WAEmuWnd->m_hWnd, WM_COMMAND, WINAMP_OPTIONS_PLEDIT, 0);
}

void CMainWnd::OnPopupMenuPrevious()
{
	::SendMessage(WAEmuWnd->m_hWnd, WM_COMMAND, WINAMP_BUTTON1, 0);
}

void CMainWnd::OnPopupMenuRemainingTime()
{
	::SendMessage(WAEmuWnd->m_hWnd, WM_COMMAND, WINAMP_OPTIONS_REMAINING, 0);
}

void CMainWnd::OnPopupMenuSettingsAdvanced()
{
	SettingsDialog = SettingsDialog->getInstance(IDD_SETTINGS, pMainFrame);
}

void CMainWnd::OnPopupMenuSettingsAfterEndOfThePlaybackExitStp()
{
	ShutdownAfterEnd = false;
	ExitAfterEnd = !ExitAfterEnd;
	if (ExitAfterEnd)
	{
		if (m_Repeat || m_RepeatTrack)
		{
			m_Repeat = m_RepeatTrack = 0;
			MessageBox(m_Helper.MyLoadString(IDS_AUTOEXIT), _T("STP"), MB_OK | MB_ICONWARNING);
		}
	}
}

void CMainWnd::OnPopupMenuSettingsAfterEndOfThePlaybackTurnOffTheComputer()
{
	ShutdownAfterEnd = !ShutdownAfterEnd;
	ExitAfterEnd = false;
	if (ShutdownAfterEnd)
	{
		if (m_Repeat || m_RepeatTrack)
		{
			m_Repeat = m_RepeatTrack = 0;
			MessageBox(m_Helper.MyLoadString(IDS_AUTOEXIT), _T("STP"), MB_OK | MB_ICONWARNING);
		}
	}
}

void CMainWnd::OnPopupMenuSettingsRandomlySelectFirstTrack()
{
	if (RandomFirst == 1)
		RandomFirst = 0;
	else
		RandomFirst = 1;
}

void CMainWnd::OnPopupMenuSettingsRepeat()
{
	if (m_Repeat == 1)
		m_Repeat = 0;
	else
	{
		m_Repeat = 1;m_RepeatTrack = 0;
	}
}

void CMainWnd::OnPopupMenuSettingsRepeatTrack()
{
	::SendMessage(WAEmuWnd->m_hWnd, WM_COMMAND, WINAMP_FILE_REPEAT, 0);
}

void CMainWnd::OnPopupMenuSettingsShuffleMode()
{
	::SendMessage(WAEmuWnd->m_hWnd, WM_COMMAND, WINAMP_FILE_SHUFFLE, 0);
}

void CMainWnd::OnPopupMenuSettingsStopAfterEachAlbum()
{
	if (AlbumStop)
		AlbumStop = false;
	else
		AlbumStop = true;
}

void CMainWnd::OnPopupMenuSeek()
{
	if (SeekDialog == NULL || SeekDialog != NULL && SeekDialog->m_hWnd == NULL)
		SeekDialog = SeekDialog->getInstance(IDD_SEEK);
	else if (SeekDialog != NULL && SeekDialog->m_hWnd != NULL)
		SeekDialog->SaveWindowPosAndDestroy();
}

void CMainWnd::OnPopupMenuShowTotalTrackTime()
{
	if (ShowTTT)
		ShowTTT = false;
	else
		ShowTTT = true;
	UpdMBTR();
}

void CMainWnd::OnPopupMenuStop()
{
	::SendMessage(WAEmuWnd->m_hWnd, WM_COMMAND, WINAMP_BUTTON4, 0);
}


void CMainWnd::OnPopupMenuTitle()
{
	ShowingMode = 0;
	m_PlaylistManager.SetShowingMode(ShowingMode);
	UpdMBTR();
}

void CMainWnd::OnTimer(UINT nIDEvent)
{
	if (nIDEvent == DELAY_FROM_BEBORE_RESTART_TOOLTIP)
	{
		KillTimer(DELAY_FROM_BEBORE_RESTART_TOOLTIP);
		startToolTip = false;
	}
	if (nIDEvent == DELAY_FROM_TOOLTIP)
	{
		startToolTip = false;
		KillTimer(DELAY_FROM_TOOLTIP);
		Shell_NotifyIcon(NIM_MODIFY, &(nid));
		if (!m_traytip.IsVisible())
			DisplayTooltip();
		CFrameWnd::OnTimer(nIDEvent);
		return;
	}
	if (nIDEvent == LBUTTON_CLICK)
	{
		if (TraySeeking == false)
		{
			TraySeeking = true;
			KillTimer(LBUTTON_CLICK);
			if (PressAndHold == 0)
				SetTimer(LBUTTON_CLICK, 500, NULL);
			else
				SetTimer(LBUTTON_CLICK, 50, NULL);
		}
		if (PressAndHold == 0)
		{
			if (m_PlaylistManager.GetSize() > 0 && stdIn->isSeekable())
				stdIn->SetOutputTime(stdIn->GetOutputTime() - 5);
			UpdMBTR();
		}
		else
		{
			CurrentVolume = pVolume->GetCurrentVolume();
			CurrentVolume -= 2;
			if (CurrentVolume < 0)
				CurrentVolume = 0;
			pVolume->SetCurrentVolume(CurrentVolume);
			stdIn->SetVolume(CurrentVolume * 255 / 100);
			if (SeekDialog != NULL && SeekDialog->m_hWnd != NULL)
				SeekDialog->m_vol.SetPos(CurrentVolume);
		}
		CFrameWnd::OnTimer(nIDEvent);
		return;
	}
	if (nIDEvent == RBUTTON_CLICK)
	{
		if (TraySeeking == false)
		{
			TraySeeking = true;
			KillTimer(RBUTTON_CLICK);
			if (PressAndHold == 0)
				SetTimer(RBUTTON_CLICK, 500, NULL);
			else
				SetTimer(RBUTTON_CLICK, 50, NULL);
		}
		if (PressAndHold == 0)
		{
			if (m_PlaylistManager.GetSize() > 0 && stdIn->isSeekable())
				stdIn->SetOutputTime(stdIn->GetOutputTime() + 5);
			UpdMBTR();
		}
		else
		{
			CurrentVolume = pVolume->GetCurrentVolume();
			CurrentVolume += 2;
			if (CurrentVolume > 100)
				CurrentVolume = 100;
			pVolume->SetCurrentVolume(CurrentVolume);
			stdIn->SetVolume(CurrentVolume * 255 / 100);
			if (SeekDialog != NULL && SeekDialog->m_hWnd != NULL)
				SeekDialog->m_vol.SetPos(CurrentVolume);
		}
		CFrameWnd::OnTimer(nIDEvent);
		return;
	}
	if (nIDEvent == REFRESH_WINDOW)
	{
		if (Created)
			if (::SendMessage(WAEmuWnd->m_hWnd, WM_WA_IPC, 0, IPC_ISPLAYING) == PLAYER_MODE_PLAYING)
				UpdMBTR(false);
		CFrameWnd::OnTimer(nIDEvent);
		return;
	}

	if (nIDEvent == LBUTTON_UP)
	{
		KillTimer(LBUTTON_UP);
		switch (lbs)
		{
		case 1:
			ClickAction(Clicks1);
			break;
		case 2:
			ClickAction(Clicks2);
			break;
		case 3:
			ClickAction(Clicks3);
			break;
		case 4:
			ClickAction(Clicks4);
			break;
		}
		lbs = 0;
		CFrameWnd::OnTimer(nIDEvent);
		return;
	}
	if (nIDEvent == RBUTTON_UP)
	{
		KillTimer(RBUTTON_UP);
		switch (rbs)
		{
		case 1:
			ClickAction(RClicks1);
			break;
		case 2:
			ClickAction(RClicks2);
			break;
		case 3:
			ClickAction(RClicks3);
			break;
		case 4:
			ClickAction(RClicks4);
			break;
		}
		rbs = 0;
		CFrameWnd::OnTimer(nIDEvent);
		return;
	}

	if (nIDEvent == DELAY_FROM_EXP)
	{
		KillTimer(DELAY_FROM_EXP);
		AddFromExp = false;
		CFrameWnd::OnTimer(nIDEvent);
		return;
	}

	if (nIDEvent == 11)
	{
		if (W2Fenabled)
			PerformW2File(false);
		CFrameWnd::OnTimer(nIDEvent);
		return;
	}

	if (nIDEvent == AUTOMATION_START)
	{
		if (AutEvery)
		{
			m_PlaylistManager.PerformAutomation();
			if (AutSaveset)
				SaveSettings();
		}
		CFrameWnd::OnTimer(nIDEvent);
		return;
	}

	if (nIDEvent == HIDE_TOOLTIP)
	{
		KillTimer(HIDE_TOOLTIP);
		HideTooltip();
		CFrameWnd::OnTimer(nIDEvent);
		return;
	}

	CFrameWnd::OnTimer(nIDEvent);
}

void CMainWnd::OnTrackMenu(UINT nID)
{
	Current = nID - IDM_TRACKS;
	if (::SendMessage(WAEmuWnd->m_hWnd, WM_WA_IPC, 0, IPC_ISPLAYING) != PLAYER_MODE_CLOSE)
		::SendMessage(WAEmuWnd->m_hWnd, WM_COMMAND, WINAMP_BUTTON4, 0);
	bool first = true;  	 
	UINT q = 0;
	do
	{
		q++;
		if (!first)
		{
			if (ShuffleMode)
				Current = MyRandom(true);
			else
			{
				UINT b, e; 
				GetAlbumRange(&b, &e);
				if (Current == e)
					Current = b;
				else
					Current++;
			}
		}
		else
			first = false;
		break;
	}
	while (q < (UINT)m_PlaylistManager.GetSize() * 2);

	enqueueFileWithMetaStruct eFile;
	eFile.filename = (TCHAR *)(LPCTSTR)m_PlaylistManager.GetPlaylist(pMainFrame->Current);
	::SendMessage(WAEmuWnd->m_hWnd, IPC_PLAYFILE, 0, (WPARAM)&eFile);
	UpdMBTR();

	if (PlaylistDlg != NULL && PlaylistDlg->m_hWnd != NULL)
		PlaylistDlg->m_playlistat.RedrawWindow(NULL, NULL, RDW_INVALIDATE | RDW_UPDATENOW);
}

void CMainWnd::OnUpdatePopupMenuArtistTitle(CCmdUI* pCmdUI)
{
	if (ShowingMode == 1)
		pCmdUI->SetCheck(1);
	else
		pCmdUI->SetCheck(0);
}

void CMainWnd::OnUpdatePopupMenuDockToActiveWindow(CCmdUI* pCmdUI)
{
	if (DockSD)
		pCmdUI->SetCheck(1);
	else
		pCmdUI->SetCheck(0);
}

void CMainWnd::OnUpdatePopupMenuElapsedTime(CCmdUI* pCmdUI)
{
	if (ShowRemainingTime == 1)
		pCmdUI->SetCheck(0);
	else
		pCmdUI->SetCheck(1);
}

void CMainWnd::OnUpdatePopupMenuEqualizer(CCmdUI* pCmdUI)
{
	if (EqualizerDlg != NULL && EqualizerDlg->m_hWnd != NULL)
		pCmdUI->SetCheck(1);
	else
		pCmdUI->SetCheck(0);
}

void CMainWnd::OnUpdatePopupMenuFileInfo(CCmdUI* pCmdUI)
{
	if (m_PlaylistManager.GetSize() == 0)
		pCmdUI->Enable(false);
	else
		pCmdUI->Enable(true);
	if (MP3InfoDlg != NULL && MP3InfoDlg->m_hWnd != NULL)
		pCmdUI->SetCheck(1);
	else
		pCmdUI->SetCheck(0);
}

void CMainWnd::OnUpdatePopupMenuFilename(CCmdUI* pCmdUI)
{
	if (ShowingMode == 2)
		pCmdUI->SetCheck(1);
	else
		pCmdUI->SetCheck(0);
}

void CMainWnd::OnUpdatePopupMenuPlaylistEditor(CCmdUI* pCmdUI)
{
	if (PlaylistDlg != NULL && PlaylistDlg->m_hWnd != NULL)
		pCmdUI->SetCheck(1);
	else
		pCmdUI->SetCheck(0);
}

void CMainWnd::OnUpdatePopupMenuRemainingTime(CCmdUI* pCmdUI)
{
	if (ShowRemainingTime == 1)
		pCmdUI->SetCheck(1);
	else
		pCmdUI->SetCheck(0);
}

void CMainWnd::OnUpdatePopupMenuSeek(CCmdUI* pCmdUI)
{
	if (SeekDialog != NULL && SeekDialog->m_hWnd != NULL)
		pCmdUI->SetCheck(1);
	else
		pCmdUI->SetCheck(0);
}

void CMainWnd::OnUpdatePopupMenuSettingsAfterEndOfThePlaybackExitStp(CCmdUI* pCmdUI)
{
	if (ExitAfterEnd)
		pCmdUI->SetCheck(1);
	else
		pCmdUI->SetCheck(0);
}

void CMainWnd::OnUpdatePopupMenuSettingsAfterEndOfThePlaybackTurnOffComputer(CCmdUI* pCmdUI)
{
	if (ShutdownAfterEnd)
		pCmdUI->SetCheck(1);
	else
		pCmdUI->SetCheck(0);
}

void CMainWnd::OnUpdatePopupMenuSettingsRandomlySelectFirstTrack(CCmdUI* pCmdUI)
{
	if (RandomFirst == 1)
		pCmdUI->SetCheck(1);
	else
		pCmdUI->SetCheck(0);
}

void CMainWnd::OnUpdatePopupMenuSettingsRepeat(CCmdUI* pCmdUI)
{
	if (m_Repeat == 1)
		pCmdUI->SetCheck(1);
	else
		pCmdUI->SetCheck(0);
	if (ExitAfterEnd || ShutdownAfterEnd)
		pCmdUI->Enable(false);
	else
		pCmdUI->Enable(true);
}

void CMainWnd::OnUpdatePopupMenuSettingsRepeatTrack(CCmdUI* pCmdUI)
{
	if (m_RepeatTrack == 1)
		pCmdUI->SetCheck(1);
	else
		pCmdUI->SetCheck(0);
	if (ExitAfterEnd || ShutdownAfterEnd)
		pCmdUI->Enable(false);
	else
		pCmdUI->Enable(true);
}

void CMainWnd::OnUpdatePopupMenuSettingsShuffleMode(CCmdUI* pCmdUI)
{
	if (ShuffleMode == 1)
		pCmdUI->SetCheck(1);
	else
		pCmdUI->SetCheck(0);
}

void CMainWnd::OnUpdatePopupMenuSettingsStopAfterEachAlbum(CCmdUI* pCmdUI)
{
	if (AlbumStop)
		pCmdUI->SetCheck(1);
	else
		pCmdUI->SetCheck(0);
}

void CMainWnd::OnUpdatePopupMenuShowTotalTrackTime(CCmdUI* pCmdUI)
{
	if (ShowTTT)
		pCmdUI->SetCheck(1);
	else
		pCmdUI->SetCheck(0);
}

void CMainWnd::OnUpdatePopupMenuTitle(CCmdUI* pCmdUI)
{
	if (ShowingMode == 0)
		pCmdUI->SetCheck(1);
	else
		pCmdUI->SetCheck(0);
}

void CMainWnd::OpenFile(CString fname, bool Add, bool ForceStart)
{
	UINT PLLPrev = m_PlaylistManager.GetSize();
	if (!Add)
	{
		m_PlaylistManager.StopReadAll();
		m_PlaylistManager.StopPerformAutomation();
		m_PlaylistManager.RemoveAll();
		Current = 0;
	}
	if (::SendMessage(WAEmuWnd->m_hWnd, WM_WA_IPC, 0, IPC_ISPLAYING) != PLAYER_MODE_CLOSE)
		::SendMessage(WAEmuWnd->m_hWnd, WM_COMMAND, WINAMP_BUTTON4, 0);
	m_PlaylistManager.Add(fname);
	if (!Add || PLLPrev == 0)
	{
		if (AutoPlay || ForceStart)
		{
			enqueueFileWithMetaStruct eFile;
			eFile.filename = (TCHAR *)(LPCTSTR)fname;
			::SendMessage(WAEmuWnd->m_hWnd, IPC_PLAYFILE, 0, (WPARAM)&eFile);
		}
		else if (::SendMessage(WAEmuWnd->m_hWnd, WM_WA_IPC, 0, IPC_ISPLAYING) != PLAYER_MODE_CLOSE)
			::SendMessage(WAEmuWnd->m_hWnd, WM_COMMAND, WINAMP_BUTTON4, 0);
	}
	UpdMBTR();
	if (PlaylistDlg != NULL && PlaylistDlg->m_hWnd != NULL)
	{
		PlaylistDlg->FillPlaylist();
		PlaylistDlg->ShowFileName();
	}
}

unsigned long __stdcall OpenFolderThread(OpenPlayListStruct* playListStruct)
{
	pMainFrame->OpenFolderEngine(playListStruct->fname, playListStruct->RandomFirst, playListStruct->Add, playListStruct->ForceStart);
	ExitThread(0);
	return 0;
}

void CMainWnd::OpenFolderEngine(CString theDir, bool RandomFirst, bool Add, bool ForceStart)
{
	UINT PLLPrev = m_PlaylistManager.GetSize();
	if (theDir[theDir.GetLength() - 1] != '\\')
		theDir += "\\";

	if (!Add)
	{
		m_PlaylistManager.StopReadAll();
		m_PlaylistManager.StopPerformAutomation();
		m_PlaylistManager.RemoveAll();
		pMainFrame->Current = 0;
		if (::SendMessage(WAEmuWnd->m_hWnd, WM_WA_IPC, 0, IPC_ISPLAYING) != PLAYER_MODE_CLOSE)
			::SendMessage(WAEmuWnd->m_hWnd, WM_COMMAND, WINAMP_BUTTON4, 0);
	}

	pMainFrame->FindMP3Files(theDir);      

	if (m_PlaylistManager.GetSize() == 0)
	{
		if (MP3InfoDlg != NULL && MP3InfoDlg->m_hWnd != NULL)
			MP3InfoDlg->SaveWindowPosAndDestroy();
		if (PlaylistDlg != NULL && PlaylistDlg->m_hWnd != NULL)
		{
			PlaylistDlg->FillPlaylist();
			PlaylistDlg->ShowFileName();
		}
		pMainFrame->UpdMBTR();
		return;
	}

	pMainFrame->SortRec(Add ? PLLPrev : 0, m_PlaylistManager.GetSize() - 1);
	if ((RandomFirst || pMainFrame->ShuffleMode) && !(Add && PLLPrev > 0))
		pMainFrame->FirstInMyRandom = pMainFrame->Current = pMainFrame->MyRandom(true, true);
	else if (!Add || PLLPrev == 0)
	{
		pMainFrame->Current = 0;pMainFrame->FirstInMyRandom = 0xFFFFFFFF;
	}

	if (PlaylistDlg != NULL && PlaylistDlg->m_hWnd != NULL)
	{
		PlaylistDlg->FillPlaylist();
		PlaylistDlg->ShowFileName();
	}

	if (!Add || PLLPrev == 0)
	{
		bool first = true;
		int q = 0;
		do
		{
			if (!first)
			{
				if (pMainFrame->Current < (UINT)m_PlaylistManager.GetSize() - 1)
					pMainFrame->Current++;
				else
				{
					pMainFrame->Current = 0;
					q++;
					if (pMainFrame->m_Repeat != 1)
						break;
				}
			}
			else
				first = false;
			if (pMainFrame->AutoPlay || ForceStart)
			{
				enqueueFileWithMetaStruct eFile;
				eFile.filename = (TCHAR *)(LPCTSTR)m_PlaylistManager.GetPlaylist(pMainFrame->Current);
				::SendMessage(WAEmuWnd->m_hWnd, IPC_PLAYFILE, 0, (WPARAM)&eFile);
			}
			else if (::SendMessage(WAEmuWnd->m_hWnd, WM_WA_IPC, 0, IPC_ISPLAYING) != PLAYER_MODE_CLOSE)
				::SendMessage(WAEmuWnd->m_hWnd, WM_COMMAND, WINAMP_BUTTON4, 0);
			break;
		}
		while (q < 2);
	}
	m_PlaylistManager.ReadAllTags();
	pMainFrame->UpdMBTR();
}

unsigned long __stdcall OpenPlaylistThread(OpenPlayListStruct* playListStruct)
{
	pMainFrame->OpenPlaylist(playListStruct->fname, playListStruct->RandomFirst, playListStruct->Add, playListStruct->ForceStart);
	ExitThread(0);
	return 0;
}
void CMainWnd::OpenPlaylistEngine(CString fname, bool RandomFirst, bool Add, bool ForceStart)
{
	UINT PLLPrev = m_PlaylistManager.GetSize();
	CStdioFile plfile;
	if (!plfile.Open(fname, CFile::modeRead | CFile::shareDenyNone))
	{
		return;
	}
	if (!Add)
	{
		m_PlaylistManager.StopReadAll();
		m_PlaylistManager.StopPerformAutomation();
		m_PlaylistManager.RemoveAll();
		pMainFrame->Current = 0;
		if (::SendMessage(WAEmuWnd->m_hWnd, WM_WA_IPC, 0, IPC_ISPLAYING) != PLAYER_MODE_CLOSE)
			::SendMessage(WAEmuWnd->m_hWnd, WM_COMMAND, WINAMP_BUTTON4, 0);
	}

	bool ifpls = ((fname.Right(4)).CompareNoCase(_T(".PLS")) == 0);

	CString buf;
	CString CurPath = fname.Left(fname.ReverseFind('\\') + 1);
	CString CurDisk = fname.Left(fname.Find(':') + 1);
	CachedTag nTag;
	while (plfile.ReadString(buf))
	{
		buf.TrimLeft();buf.TrimRight();
		if (buf.Left(4) == "#EXT" || buf == "")
			continue;
		if (buf.Left(5) == "#TAG=" && m_PlaylistManager.GetSize() > 0 && !ifpls)
			continue;
		if (ifpls)
		{
			if (buf.Left(4).CompareNoCase(_T("file")) == 0)
				buf = buf.Mid(buf.Find('=') + 1);
			else
				continue;
		}
		if (buf.Find(_T("://")) != 0)
		{
			//  		  buf.Replace(" ", "%20");
			nTag.SetPlaylist(buf);
		}
		else if (buf[1] == ':' || buf[0] == '\\' && buf[1] == '\\')
			nTag.SetPlaylist(buf);
		else if (buf[0] == '\\')
			nTag.SetPlaylist(CurDisk + buf);
		else
			nTag.SetPlaylist(CurPath + buf);
		m_PlaylistManager.Add(nTag);
	}
	plfile.Close();

	if (m_PlaylistManager.GetSize() == 0)
	{
		if (MP3InfoDlg != NULL && MP3InfoDlg->m_hWnd != NULL)
			MP3InfoDlg->SaveWindowPosAndDestroy();
		if (PlaylistDlg != NULL && PlaylistDlg->m_hWnd != NULL)
		{
			PlaylistDlg->FillPlaylist();
			PlaylistDlg->ShowFileName();
		}
		pMainFrame->UpdMBTR();
		return;
	}
	if ((RandomFirst || pMainFrame->ShuffleMode) && !(Add && PLLPrev > 0))
		pMainFrame->FirstInMyRandom = pMainFrame->Current = pMainFrame->MyRandom(true, true);
	else if (!Add || PLLPrev == 0)
	{
		pMainFrame->Current = 0;pMainFrame->FirstInMyRandom = 0xFFFFFFFF;
	}

	if (PlaylistDlg != NULL && PlaylistDlg->m_hWnd != NULL)
	{
		PlaylistDlg->FillPlaylist();
		PlaylistDlg->ShowFileName();
	}

	if (!Add || PLLPrev == 0)
	{
		bool first = true;
		int q = 0;
		do
		{
			if (!first)
			{
				if (pMainFrame->Current < (UINT)m_PlaylistManager.GetSize() - 1)
					pMainFrame->Current++;
				else
				{
					pMainFrame->Current = 0;
					q++;
					if (pMainFrame->m_Repeat != 1)
						break;
				}
			}
			else
				first = false;
			if (pMainFrame->AutoPlay || ForceStart)
			{
				enqueueFileWithMetaStruct eFile;
				eFile.filename = (TCHAR *)(LPCTSTR)m_PlaylistManager.GetPlaylist(pMainFrame->Current);
				::SendMessage(WAEmuWnd->m_hWnd, IPC_PLAYFILE, 0, (WPARAM)&eFile);
			}
			else if (::SendMessage(WAEmuWnd->m_hWnd, WM_WA_IPC, 0, IPC_ISPLAYING) != PLAYER_MODE_CLOSE)
				::SendMessage(WAEmuWnd->m_hWnd, WM_COMMAND, WINAMP_BUTTON4, 0);
			break;
		}
		while (q < 2);
	}
	pMainFrame->UpdMBTR();
}

void CMainWnd::OpenFolder(CString theDir, bool RandomFirst, bool Add, bool ForceStart)
{
	myPlayListStruct.fname = theDir;
	myPlayListStruct.RandomFirst = RandomFirst;
	myPlayListStruct.Add = Add;
	myPlayListStruct.ForceStart = ForceStart;
	OpenFolderThreadHan = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)OpenFolderThread, &myPlayListStruct, 0, &OpenFolderThreadID);
	SetThreadPriority(OpenFolderThreadHan, THREAD_PRIORITY_LOWEST);
}

void CMainWnd::OpenPlaylist(CString fname, bool RandomFirst, bool Add, bool ForceStart)
{
	myPlayListStruct.fname = fname;
	myPlayListStruct.RandomFirst = RandomFirst;
	myPlayListStruct.Add = Add;
	myPlayListStruct.ForceStart = ForceStart;
	OpenPlaylistThreadHan = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)OpenPlaylistThread, &myPlayListStruct, 0, &OpenPlaylistThreadID);
	SetThreadPriority(OpenPlaylistThreadHan, THREAD_PRIORITY_LOWEST);
}

void CMainWnd::OpenRecent(int num)
{
	if (Recent[num] == "")
		return;
	if (Recent[num].Left(7).CompareNoCase(_T("http://")) == 0 || Recent[num].Left(6).CompareNoCase(_T("ftp://")) == 0)
	{
		OpenUrl(Recent[num]);
		return;
	}
	CFileFind(finder);
	if (finder.FindFile(Recent[num]))
	{
		finder.FindNextFile();
		if (finder.IsDirectory())
			OpenFolder(Recent[num], RandomFirst == 1);
		else
		{
			if ((Recent[num].Mid(Recent[num].ReverseFind('.') + 1, 3)).CompareNoCase(_T("M3U")) == 0 || (Recent[num].Mid(Recent[num].ReverseFind('.') + 1, 3)).CompareNoCase(_T("PLS")) == 0)
				OpenPlaylist(Recent[num], RandomFirst == 1);
			else
				OpenFile(Recent[num]);
		}
		CString buf = Recent[num];
		for (int i = num; i > 0; i--)
			Recent[i] = Recent[i - 1];
		Recent[0] = buf;
	}
	else
	{
		for (int i = num; i < 9; i++)
			Recent[i] = Recent[i + 1];
		Recent[9] = "";
	}
}

void CMainWnd::OpenUrl(CString m_URL, bool Add)
{
	CRect rect;

	UINT PLLPrev = m_PlaylistManager.GetSize();
	if (!Add)
	{
		m_PlaylistManager.StopReadAll();
		m_PlaylistManager.StopPerformAutomation();
		m_PlaylistManager.RemoveAll();
		Current = 0;
		if (::SendMessage(WAEmuWnd->m_hWnd, WM_WA_IPC, 0, IPC_ISPLAYING) != PLAYER_MODE_CLOSE)
			::SendMessage(WAEmuWnd->m_hWnd, WM_COMMAND, WINAMP_BUTTON4, 0);
	}
	m_PlaylistManager.Add(m_URL);
	if (!Add || PLLPrev == 0)
	{
		if (AutoPlay)
		{
			enqueueFileWithMetaStruct eFile;
			eFile.filename = (TCHAR *)(LPCTSTR)m_URL;
			::SendMessage(WAEmuWnd->m_hWnd, IPC_PLAYFILE, 0, (WPARAM)&eFile);
		}
	}
	UpdMBTR();
	if (PlaylistDlg != NULL && PlaylistDlg->m_hWnd != NULL)
	{
		PlaylistDlg->FillPlaylist();
		PlaylistDlg->ShowFileName();
	}
}

void CMainWnd::PerformW2File(bool ex)
{
	if (W2Ffile != "")
	{
		CStdioFile f;
		if (f.Open(W2Ffile, CFile::modeCreate | CFile::modeWrite))
		{
			CString buf;
			if (!ex)
			{
				switch (::SendMessage(WAEmuWnd->m_hWnd, WM_WA_IPC, 0, IPC_ISPLAYING))
				{
				case PLAYER_MODE_PLAYING :
					buf = W2Fplay;
					break;
				case PLAYER_MODE_PAUSED :
					buf = W2Fpause;
					break;
				case PLAYER_MODE_CLOSE :
					buf = W2Fstop;
					break;
				}
				buf.Replace(_T("%1"), m_PlaylistManager.GetArtist(Current));
				buf.Replace(_T("%2"), m_PlaylistManager.GetTitle(Current));
				buf.Replace(_T("%3"), m_PlaylistManager.GetAlbum(Current));
				buf.Replace(_T("%4"), m_PlaylistManager.GetYear(Current));
				buf.Replace(_T("%7"), m_PlaylistManager.GetFilename(Current));
			}
			else
				buf = W2Fexit;
			//  		  if(W2Foem)
			//  			  buf.AnsiToOem();
			buf.TrimLeft(); 
			buf.TrimRight();
			f.WriteString(buf);
			f.Close();
			if (W2Fdel && ex)
				::DeleteFile(W2Ffile);
		}
	}
}

void CMainWnd::RegHotKeys()
{
	if (m_PlayKey != 0)
		RegisterHotKey(m_hWnd, 101, m_PlayMod, m_PlayKey);
	if (m_StopKey != 0)
		RegisterHotKey(m_hWnd, 102, m_StopMod, m_StopKey);
	if (m_PrevKey != 0)
		RegisterHotKey(m_hWnd, 103, m_PrevMod, m_PrevKey);
	if (m_NextKey != 0)
		RegisterHotKey(m_hWnd, 104, m_NextMod, m_NextKey);
	if (m_VDownKey != 0)
		RegisterHotKey(m_hWnd, 202, m_VDownMod, m_VDownKey);
	if (m_VUpKey != 0)
		RegisterHotKey(m_hWnd, 201, m_VUpMod, m_VUpKey);
	if (m_OpenKey1 != 0)
		RegisterHotKey(m_hWnd, 301, m_OpenMod1, m_OpenKey1);
	if (m_OpenKey2 != 0)
		RegisterHotKey(m_hWnd, 302, m_OpenMod2, m_OpenKey2);
	if (m_OpenKey3 != 0)
		RegisterHotKey(m_hWnd, 303, m_OpenMod3, m_OpenKey3);
	if (m_MinibarKey != 0)
		RegisterHotKey(m_hWnd, 501, m_MinibarMod, m_MinibarKey);
	if (m_MP3InfoKey != 0)
		RegisterHotKey(m_hWnd, 901, m_MP3InfoMod, m_MP3InfoKey);
	if (m_PlaylistKey != 0)
		RegisterHotKey(m_hWnd, 801, m_PlaylistMod, m_PlaylistKey);
	if (m_EqualizerKey != 0)
		RegisterHotKey(m_hWnd, 601, m_EqualizerMod, m_EqualizerKey);
	if (m_RepeatKey != 0)
		RegisterHotKey(m_hWnd, 701, m_RepeatMod, m_RepeatKey);
	if (m_ShuffleKey != 0)
		RegisterHotKey(m_hWnd, 401, m_ShuffleMod, m_ShuffleKey);
	if (m_AlbummodeKey != 0)
		RegisterHotKey(m_hWnd, 1601, m_AlbummodeMod, m_AlbummodeKey);
	if (m_ExitKey != 0)
		RegisterHotKey(m_hWnd, 1001, m_ExitMod, m_ExitKey);
	if (m_NPKey != 0)
		RegisterHotKey(m_hWnd, 1201, m_NPMod, m_NPKey);
	if (m_ForwKey != 0)
		RegisterHotKey(m_hWnd, 1301, m_ForwMod, m_ForwKey);
	if (m_BackKey != 0)
		RegisterHotKey(m_hWnd, 1302, m_BackMod, m_BackKey);
	if (m_TrayIconKey != 0)
		RegisterHotKey(m_hWnd, 1401, m_TrayIconMod, m_TrayIconKey);
	if (m_MenuKey != 0)
		RegisterHotKey(m_hWnd, 1501, m_MenuMod, m_MenuKey);
	if (m_TracksKey != 0)
		RegisterHotKey(m_hWnd, 1502, m_TracksMod, m_TracksKey);
	if (m_AlbumKey != 0)
		RegisterHotKey(m_hWnd, 1503, m_AlbumMod, m_AlbumKey);
	if (m_AutomationKey != 0)
		RegisterHotKey(m_hWnd, 1701, m_AutomationMod, m_AutomationKey);
}

CString CMainWnd::ReplaceAmpersand(CString s)
{
	s.Replace(_T("&"), _T("&&"));
	return s;
}

void CMainWnd::ResetSkin()
{
	HICON m_hIcon;

	if (!stdIn->IsPaused())
		m_hIcon = PlayIcon;
	else
		m_hIcon = PauseIcon;

	nid.hIcon = m_hIcon;
	if (IconShown)
		Shell_NotifyIcon(NIM_MODIFY, &nid);
}

void CMainWnd::SaveSettings()
{
	theApp.WriteProfileString(_T("Preferences"), _T("Language"), Language);

	theApp.WriteProfileInt(_T("Preferences"), _T("InfoX"), InfoX);
	theApp.WriteProfileInt(_T("Preferences"), _T("InfoY"), InfoY);
	theApp.WriteProfileInt(_T("Preferences"), _T("SBDockR"), SBDockR);
	theApp.WriteProfileInt(_T("Preferences"), _T("SBDockB"), SBDockB);
	theApp.WriteProfileInt(_T("Preferences"), _T("SeekBarX"), SeekBarX);
	theApp.WriteProfileInt(_T("Preferences"), _T("SeekBarY"), SeekBarY);
	theApp.WriteProfileInt(_T("Preferences"), _T("SeekBarW"), SeekBarW);
	theApp.WriteProfileInt(_T("Preferences"), _T("EqualizerX"), EqualizerX);
	theApp.WriteProfileInt(_T("Preferences"), _T("EqualizerY"), EqualizerY);
	theApp.WriteProfileInt(_T("Preferences"), _T("PlaylistX"), PlaylistX);
	theApp.WriteProfileInt(_T("Preferences"), _T("PlaylistY"), PlaylistY);
	theApp.WriteProfileInt(_T("Preferences"), _T("PlaylistW"), PlaylistW);
	theApp.WriteProfileInt(_T("Preferences"), _T("PlaylistH"), PlaylistH);

	theApp.WriteProfileInt(_T("Preferences"), _T("Clicks1"), Clicks1);
	theApp.WriteProfileInt(_T("Preferences"), _T("Clicks2"), Clicks2);
	theApp.WriteProfileInt(_T("Preferences"), _T("Clicks3"), Clicks3);
	theApp.WriteProfileInt(_T("Preferences"), _T("Clicks4"), Clicks4);
	theApp.WriteProfileInt(_T("Preferences"), _T("RClicks1"), RClicks1);
	theApp.WriteProfileInt(_T("Preferences"), _T("RClicks2"), RClicks2);
	theApp.WriteProfileInt(_T("Preferences"), _T("RClicks3"), RClicks3);
	theApp.WriteProfileInt(_T("Preferences"), _T("RClicks4"), RClicks4);

	theApp.WriteProfileString(_T("Preferences"), _T("CurrentSkin"), CurrentSkin);

	theApp.WriteProfileString(_T("Preferences"), _T("RNTemplate"), RNTemplate);

	theApp.WriteProfileInt(_T("Preferences"), _T("DockMinibar"), DockSD ? 1 : 0);

	theApp.WriteProfileInt(_T("Preferences"), _T("SMTracks"), SMTracks ? 1 : 0);
	theApp.WriteProfileInt(_T("Preferences"), _T("SMAlbum"), SMAlbum ? 1 : 0);
	theApp.WriteProfileInt(_T("Preferences"), _T("SMPrevNext"), SMPrevNext ? 1 : 0);
#ifndef _UNICODE
	theApp.WriteProfileInt(_T("Preferences"), _T("WAEmu"), WAEmu ? 1 : 0);
#endif
	theApp.WriteProfileInt(_T("Preferences"), _T("PressAndHold"), PressAndHold);

	theApp.WriteProfileInt(_T("Preferences"), _T("ShowRemainingTime"), ShowRemainingTime);
	theApp.WriteProfileInt(_T("Preferences"), _T("ShowTTT"), ShowTTT ? 1 : 0);

	theApp.WriteProfileInt(_T("Preferences"), _T("ShowGradientBar"), ShowGradientBar ? 1 : 0);
	theApp.WriteProfileInt(_T("Preferences"), _T("GraphicalButtons"), GraphicalButtons ? 1 : 0);

	theApp.WriteProfileInt(_T("Preferences"), _T("TPC"), TPC);

	theApp.WriteProfileInt(_T("Preferences"), _T("ExtPlaylistFormat"), ExtPlaylistFormat);

	theApp.WriteProfileInt(_T("Preferences"), _T("AlbumStop"), AlbumStop ? 1 : 0);

	theApp.WriteProfileInt(_T("Preferences"), _T("ShowingMode"), ShowingMode);   

	theApp.WriteProfileInt(_T("Preferences"), _T("NoClicks"), NoClicks);

	theApp.WriteProfileInt(_T("Preferences"), _T("PlaylistOnTop"), PlaylistOnTop);
	theApp.WriteProfileInt(_T("Preferences"), _T("EQOnTop"), EQOnTop);
	theApp.WriteProfileInt(_T("Preferences"), _T("InfoOnTop"), InfoOnTop);

	theApp.WriteProfileInt(_T("Preferences"), _T("AutoPlay"), AutoPlay);
	theApp.WriteProfileInt(_T("Preferences"), _T("RandomFirst"), RandomFirst);

	theApp.WriteProfileInt(_T("Preferences"), _T("Gap"), GapBtwTracks);

	theApp.WriteProfileInt(_T("Preferences"), _T("NPKey"), m_NPKey);
	theApp.WriteProfileInt(_T("Preferences"), _T("NPMod"), m_NPMod);
	theApp.WriteProfileInt(_T("Preferences"), _T("ExitKey"), m_ExitKey);
	theApp.WriteProfileInt(_T("Preferences"), _T("ExitMod"), m_ExitMod);
	theApp.WriteProfileInt(_T("Preferences"), _T("ForwKey"), m_ForwKey);
	theApp.WriteProfileInt(_T("Preferences"), _T("ForwMod"), m_ForwMod);
	theApp.WriteProfileInt(_T("Preferences"), _T("BackKey"), m_BackKey);
	theApp.WriteProfileInt(_T("Preferences"), _T("BackMod"), m_BackMod);
	theApp.WriteProfileInt(_T("Preferences"), _T("PlayKey"), m_PlayKey);
	theApp.WriteProfileInt(_T("Preferences"), _T("PlayMod"), m_PlayMod);
	theApp.WriteProfileInt(_T("Preferences"), _T("StopKey"), m_StopKey);
	theApp.WriteProfileInt(_T("Preferences"), _T("StopMod"), m_StopMod);
	theApp.WriteProfileInt(_T("Preferences"), _T("PrevKey"), m_PrevKey);
	theApp.WriteProfileInt(_T("Preferences"), _T("PrevMod"), m_PrevMod);
	theApp.WriteProfileInt(_T("Preferences"), _T("NextKey"), m_NextKey);
	theApp.WriteProfileInt(_T("Preferences"), _T("NextMod"), m_NextMod);
	theApp.WriteProfileInt(_T("Preferences"), _T("VUpKey"), m_VUpKey);
	theApp.WriteProfileInt(_T("Preferences"), _T("VUpMod"), m_VUpMod);
	theApp.WriteProfileInt(_T("Preferences"), _T("VDownKey"), m_VDownKey);
	theApp.WriteProfileInt(_T("Preferences"), _T("VDownMod"), m_VDownMod);

	theApp.WriteProfileInt(_T("Preferences"), _T("OpenKey1"), m_OpenKey1);
	theApp.WriteProfileInt(_T("Preferences"), _T("OpenMod1"), m_OpenMod1);
	theApp.WriteProfileInt(_T("Preferences"), _T("OpenKey2"), m_OpenKey2);
	theApp.WriteProfileInt(_T("Preferences"), _T("OpenMod2"), m_OpenMod2);
	theApp.WriteProfileInt(_T("Preferences"), _T("OpenKey3"), m_OpenKey3);
	theApp.WriteProfileInt(_T("Preferences"), _T("OpenMod3"), m_OpenMod3);
	theApp.WriteProfileInt(_T("Preferences"), _T("MinibarKey"), m_MinibarKey);
	theApp.WriteProfileInt(_T("Preferences"), _T("MinibarMod"), m_MinibarMod);
	theApp.WriteProfileInt(_T("Preferences"), _T("MP3InfoKey"), m_MP3InfoKey);
	theApp.WriteProfileInt(_T("Preferences"), _T("MP3InfoMod"), m_MP3InfoMod);
	theApp.WriteProfileInt(_T("Preferences"), _T("PlstKey"), m_PlaylistKey);
	theApp.WriteProfileInt(_T("Preferences"), _T("PlstMod"), m_PlaylistMod);
	theApp.WriteProfileInt(_T("Preferences"), _T("EqualizerKey"), m_EqualizerKey);
	theApp.WriteProfileInt(_T("Preferences"), _T("EqualizerMod"), m_EqualizerMod);
	theApp.WriteProfileInt(_T("Preferences"), _T("RptKey"), m_RepeatKey);
	theApp.WriteProfileInt(_T("Preferences"), _T("RptMod"), m_RepeatMod);
	theApp.WriteProfileInt(_T("Preferences"), _T("ShflKey"), m_ShuffleKey);
	theApp.WriteProfileInt(_T("Preferences"), _T("ShflMod"), m_ShuffleMod);
	theApp.WriteProfileInt(_T("Preferences"), _T("AlbummodeKey"), m_AlbummodeKey);
	theApp.WriteProfileInt(_T("Preferences"), _T("AlbummodeMod"), m_AlbummodeMod);
	theApp.WriteProfileInt(_T("Preferences"), _T("TrayIconKey"), m_TrayIconKey);
	theApp.WriteProfileInt(_T("Preferences"), _T("TrayIconMod"), m_TrayIconMod);
	theApp.WriteProfileInt(_T("Preferences"), _T("MenuKey"), m_MenuKey);
	theApp.WriteProfileInt(_T("Preferences"), _T("MenuMod"), m_MenuMod);
	theApp.WriteProfileInt(_T("Preferences"), _T("TracksKey"), m_TracksKey);
	theApp.WriteProfileInt(_T("Preferences"), _T("TracksMod"), m_TracksMod);
	theApp.WriteProfileInt(_T("Preferences"), _T("AlbumKey"), m_AlbumKey);
	theApp.WriteProfileInt(_T("Preferences"), _T("AlbumMod"), m_AlbumMod);
	theApp.WriteProfileInt(_T("Preferences"), _T("AutomationKey"), m_AutomationKey);
	theApp.WriteProfileInt(_T("Preferences"), _T("AutomationMod"), m_AutomationMod);

	theApp.WriteProfileString(_T("Preferences"), _T("InitDir"), InitDir);

	theApp.WriteProfileString(_T("Preferences"), _T("Recent0"), Recent[0]);
	theApp.WriteProfileString(_T("Preferences"), _T("Recent1"), Recent[1]);
	theApp.WriteProfileString(_T("Preferences"), _T("Recent2"), Recent[2]);
	theApp.WriteProfileString(_T("Preferences"), _T("Recent3"), Recent[3]);
	theApp.WriteProfileString(_T("Preferences"), _T("Recent4"), Recent[4]);
	theApp.WriteProfileString(_T("Preferences"), _T("Recent5"), Recent[5]);
	theApp.WriteProfileString(_T("Preferences"), _T("Recent6"), Recent[6]);
	theApp.WriteProfileString(_T("Preferences"), _T("Recent7"), Recent[7]);
	theApp.WriteProfileString(_T("Preferences"), _T("Recent8"), Recent[8]);
	theApp.WriteProfileString(_T("Preferences"), _T("Recent9"), Recent[9]);

	theApp.WriteProfileInt(_T("Preferences"), _T("EQPreset"), EQPreset);

	theApp.WriteProfileInt(_T("Preferences"), _T("Repeat"), m_Repeat);
	theApp.WriteProfileInt(_T("Preferences"), _T("RepeatTrack"), m_RepeatTrack);
	theApp.WriteProfileInt(_T("Preferences"), _T("ShuffleMode"), ShuffleMode);
	theApp.WriteProfileInt(_T("Preferences"), _T("Preamp"), m_preamp);
	theApp.WriteProfileInt(_T("Preferences"), _T("EnableEQ"), m_EEQ);
	theApp.WriteProfileInt(_T("Preferences"), _T("EQ1"), m_s1);
	theApp.WriteProfileInt(_T("Preferences"), _T("EQ2"), m_s2);
	theApp.WriteProfileInt(_T("Preferences"), _T("EQ3"), m_s3);
	theApp.WriteProfileInt(_T("Preferences"), _T("EQ4"), m_s4);
	theApp.WriteProfileInt(_T("Preferences"), _T("EQ5"), m_s5);
	theApp.WriteProfileInt(_T("Preferences"), _T("EQ6"), m_s6);
	theApp.WriteProfileInt(_T("Preferences"), _T("EQ7"), m_s7);
	theApp.WriteProfileInt(_T("Preferences"), _T("EQ8"), m_s8);
	theApp.WriteProfileInt(_T("Preferences"), _T("EQ9"), m_s9);
	theApp.WriteProfileInt(_T("Preferences"), _T("EQ10"), m_s10);

	theApp.WriteProfileInt(_T("Preferences"), _T("AutSort"), AutSort ? 1 : 0);
	theApp.WriteProfileString(_T("Preferences"), _T("W2Ffile"), W2Ffile);
	theApp.WriteProfileString(_T("Preferences"), _T("W2Fplay"), _T("\"") + W2Fplay + _T("\""));
	theApp.WriteProfileString(_T("Preferences"), _T("W2Fpause"), _T("\"") + W2Fpause + _T("\""));
	theApp.WriteProfileString(_T("Preferences"), _T("W2Fstop"), _T("\"") + W2Fstop + _T("\""));
	theApp.WriteProfileString(_T("Preferences"), _T("W2Fexit"), _T("\"") + W2Fexit + _T("\""));
	theApp.WriteProfileInt(_T("Preferences"), _T("W2Foem"), W2Foem);
	theApp.WriteProfileInt(_T("Preferences"), _T("W2Fdel"), W2Fdel);
	theApp.WriteProfileInt(_T("Preferences"), _T("W2Fenabled"), W2Fenabled);

	theApp.WriteProfileInt(_T("Preferences"), _T("AutOnStart"), AutOnStart ? 1 : 0);
	theApp.WriteProfileInt(_T("Preferences"), _T("AutSaveset"), AutSaveset ? 1 : 0);

	theApp.WriteProfileInt(_T("Preferences"), _T("AutRemove"), AutRemove ? 1 : 0);
	theApp.WriteProfileInt(_T("Preferences"), _T("AutRemovedup"), AutRemovedup ? 1 : 0);
	theApp.WriteProfileInt(_T("Preferences"), _T("AutRescan"), AutRescan ? 1 : 0);
	theApp.WriteProfileInt(_T("Preferences"), _T("AutPeriod"), AutPeriod);
	theApp.WriteProfileInt(_T("Preferences"), _T("AutEvery"), AutEvery ? 1 : 0);

	int AutDirsCnt = AutDirs.GetSize();
	theApp.WriteProfileInt(_T("Preferences"), _T("AutDirsCnt"), AutDirsCnt);

	CString adtmp;
	for (int i = 0; i < AutDirsCnt; i++)
	{
		adtmp.Format(_T("%d"), i + 1);
		theApp.WriteProfileString(_T("Preferences"), _T("AutDir") + adtmp, AutDirs[i]);
	}

	int q;
	q = presets.GetSize();
	theApp.WriteProfileInt(_T("Preferences"), _T("Presets"), q);

	CString buf, buf2;
	for (i = 0; i < q; i++)
	{
		buf2.Format(_T("PresetName%d"), i);
		theApp.WriteProfileString(_T("Preferences"), buf2, presets[i].Left(presets[i].Find('\t')));
		buf2.Format(_T("PresetEQ%d"), i);
		theApp.WriteProfileString(_T("Preferences"), buf2, presets[i].Mid(presets[i].Find('\t') + 1));
	}

	theApp.WriteProfileInt(_T("MenuXP"), _T("ShowGradient"), showGradient ? 1 : 0);
	theApp.WriteProfileInt(_T("Preferences"), _T("showToolTip"), showToolTip ? 1 : 0);
	theApp.WriteProfileInt(_T("Preferences"), _T("autoUpdate"), autoUpdate ? 1 : 0);
	theApp.WriteProfileInt(_T("Preferences"), _T("displayConsole"), displayConsole ? 1 : 0);
	CString tmp;
	TCHAR hexa[16][2] =
	{
		{_T("0")}, {_T("1")}, {_T("2")}, {_T("3")}, {_T("4")}, {_T("5")}, {_T("6")}, {_T("7")}, {_T("8")}, {_T("9")}, {_T("A")}, {_T("B")}, {_T("C")}, {_T("D")}, {_T("E")}, {_T("F")}
	};
	tmp.Format(_T("0x%s%s%s%s%s%s"), hexa[GetBValue(startColor) / 16], hexa[GetBValue(startColor) % 16], hexa[GetGValue(startColor) / 16], hexa[GetGValue(startColor) % 16], hexa[GetRValue(startColor) / 16], hexa[GetRValue(startColor) % 16]);
	theApp.WriteProfileString(_T("MenuXP"), _T("GradientStart"), tmp);
	tmp.Format(_T("0x%s%s%s%s%s%s"), hexa[GetBValue(endColor) / 16], hexa[GetBValue(endColor) % 16], hexa[GetGValue(endColor) / 16], hexa[GetGValue(endColor) % 16], hexa[GetRValue(endColor) / 16], hexa[GetRValue(endColor) % 16]);
	theApp.WriteProfileString(_T("MenuXP"), _T("GradientEnd"), tmp);
	theApp.WriteProfileString(_T("Preferences"), _T("ToolTipText"), toolTipText);
	theApp.WriteProfileInt(_T("Preferences"), _T("ToolTipTime"), toolTipTime);
}

void CMainWnd::SaveThePlaylist()
{
	TCHAR buf2[MAX_PATH + 1];
	GetModuleFileName(theApp.m_hInstance, buf2, MAX_PATH);
	for (int i = _tcslen(buf2) - 1; i > 0; i--)
	{
		if (buf2[i] == '.')
		{
			buf2[i] = '\0';break;
		}
	}
	CString fname;
	fname = buf2;
	fname += ".m3u";

	CStdioFile plfile;

	CStringArray prev;
	CString buf;

	if (plfile.Open(fname, CFile::modeRead | CFile::shareDenyNone))
	{
		while (plfile.ReadString(buf))
		{
			buf.TrimLeft();buf.TrimRight();
			prev.Add(buf);
		}
		plfile.Close();
	}

	if (plfile.Open(fname, CFile::modeCreate | CFile::modeWrite))
	{
		UINT i;
		/*  		UINT ps=prev.GetSize();
																															for(i=0;i<ps;i++)
																																plfile.WriteString(prev[i]+"\n");
																													  */		  for (i = 0; i < (UINT)m_PlaylistManager.GetSize(); i++)
																															  {
																																  	plfile.WriteString(m_PlaylistManager.GetPlaylist(i) + "\n");
																															  	if (m_PlaylistManager.GetRead(i))
																															  	{
																															  		CString tmp;
																																	tmp.Format(_T("#TAG=%s\n"), m_PlaylistManager.GetTag(i));
																																	plfile.WriteString(tmp);
																															  	}
																															  }
		plfile.Close();
		theApp.WriteProfileInt(_T("Preferences"), _T("CurrentInPlaylist"), Current);
	}
}

void CMainWnd::SetEQ()
{
	double eq[32];

	if (m_EEQ == 0)
	{
		// unset the equalizer
		// 27/05
		Args.UseEqualizer = false;
		return;
	}
	// 27/05
	Args.UseEqualizer = true;

	// compute channel bands;

	eq[0] = m_s1;
	eq[1] = m_s2;
	eq[2] = m_s3;
	eq[3] = m_s4;
	eq[4] = m_s5;
	eq[5] = m_s6;
	eq[6] = m_s7;
	eq[7] = m_s7;
	eq[8] = m_s8;
	eq[9] = m_s8;
	eq[10] = m_s8;
	eq[11] = m_s8;
	eq[12] = m_s9;
	eq[13] = m_s9;
	eq[14] = m_s9;
	eq[15] = m_s9;
	eq[16] = m_s9;
	eq[17] = m_s9;
	eq[18] = m_s9;
	eq[19] = m_s9;
	eq[20] = m_s10;
	eq[21] = m_s10;
	eq[22] = m_s10;
	eq[23] = m_s10;
	eq[24] = m_s10;
	eq[25] = m_s10;
	eq[26] = m_s10;
	eq[27] = m_s10;
	eq[28] = m_s10;
	eq[29] = m_s10;
	eq[30] = m_s10;
	eq[31] = m_s10;

	for (int i = 0; i < 32; i++)
		equalizer[i] = (1 + eq[i] * .05) * (1 + eq[i] * .05) * (1 + m_preamp * .05) * (1 + m_preamp * .05);
}

void CMainWnd::ShowPopup(int MenuType)
{
	//TODO A corriger
	//  CGradientMenu* pPopup=new CGradientMenu(24,1);
	CMenu* pPopup;
	if (mainMenu != NULL)
	{
		delete mainMenu;
		mainMenu = new CMenu();
	}
	mainMenu->LoadMenu(IDR_MENU);
	POINT point;
	switch (MenuType)
	{
	case 0:
		point = CurMousePos; break;
	case 1:
		GetPosForMenu(&point); break;
	case 2:
		GetCursorPos(&point); break;
	}
	//TODO A corriger
	//  pPopup->Attach(mainMenu->GetSubMenu(0)->Detach());
	pPopup = mainMenu->GetSubMenu(0);
	m_Helper.translateMenu(pPopup, IDR_MENU);

	CMenu* pTempMenu;
	// Rcent
	pTempMenu = pPopup->GetSubMenu(6)->GetSubMenu(5);
	pTempMenu->DeleteMenu(0, MF_BYPOSITION);
	int i, j, k; CString rbuf;
	for (i = 0; i < 10; i++)
	{
		if (Recent[i] != "")
		{
			rbuf = Recent[i];
			if (rbuf.GetLength() > 40)
			{
				j = rbuf.ReverseFind('\\');
				k = j > 20 ? 18 : j - 3; if (k < 0)
										 	k = 0;
				rbuf = rbuf.Left(k) + "....." + rbuf.Mid(j);
			}
			pTempMenu->AppendMenu(MF_STRING, IDM_RECENT_FILES + i, ReplaceAmpersand(rbuf));
		}
	}

	// Scan drive
	pTempMenu = pPopup->GetSubMenu(6)->GetSubMenu(6);
	pTempMenu->DeleteMenu(0, MF_BYPOSITION);
	CString CDLetter;
	UINT dt;
	for (TCHAR c = 'C'; c <= 'Z'; c++)
	{
		CDLetter.Format(_T("%c"), c);
		dt = ::GetDriveType(CDLetter + ":\\");
		if (dt == DRIVE_REMOVABLE || dt == DRIVE_FIXED || dt == DRIVE_REMOTE || dt == DRIVE_CDROM)
			pTempMenu->AppendMenu(MF_STRING, IDM_DRIVES + (UINT)(c - 'C'), CDLetter);
	}

	// Menu prev next
	if (SMPrevNext && m_PlaylistManager.GetSize() > 0)
	{
		pTempMenu = pPopup->GetSubMenu(18);
		pTempMenu->DeleteMenu(0, MF_BYPOSITION);
		BuildPrevNextMenu(pTempMenu);
	}
	else
		pPopup->DeleteMenu(18, MF_BYPOSITION);

	if (m_PlaylistManager.GetSize() > 0 && m_PlaylistManager.GetSize() <= 50000)
	{
		if (SMAlbum)
		{
			pTempMenu = pPopup->GetSubMenu(8);
			pTempMenu->DeleteMenu(0, MF_BYPOSITION);
			BuildAlbumMenu(pTempMenu);
		}
		else
			pPopup->DeleteMenu(8, MF_BYPOSITION);
		if (SMTracks)
		{
			pTempMenu = pPopup->GetSubMenu(7);
			pTempMenu->DeleteMenu(0, MF_BYPOSITION);
			BuildMenu(pTempMenu);
		}
		else
			pPopup->DeleteMenu(7, MF_BYPOSITION);
	}
	else
	{
		pPopup->DeleteMenu(8, MF_BYPOSITION);
		pPopup->DeleteMenu(7, MF_BYPOSITION);
	}

	if (MenuType < 2)
	{
		if (GraphicalButtons)
		{
			pPopup->ModifyMenu(ID_POPUPMENU_PLAY, MF_OWNERDRAW | MF_BYCOMMAND, ID_POPUPMENU_PLAY);
			pPopup->ModifyMenu(ID_POPUPMENU_PAUSE, MF_OWNERDRAW | MF_BYCOMMAND, ID_POPUPMENU_PAUSE);
			pPopup->ModifyMenu(ID_POPUPMENU_STOP, MF_OWNERDRAW | MF_BYCOMMAND, ID_POPUPMENU_STOP);
			pPopup->ModifyMenu(ID_POPUPMENU_PREVIOUS, MF_OWNERDRAW | MF_BYCOMMAND, ID_POPUPMENU_PREVIOUS);
			pPopup->ModifyMenu(ID_POPUPMENU_NEXT, MF_OWNERDRAW | MF_BYCOMMAND, ID_POPUPMENU_NEXT);
		}

		if (ShowGradientBar)
		{
			pPopup->InsertMenu(0, MF_OWNERDRAW | MF_BYPOSITION, ID_POPUPMENU_PIC);
			CString buf;
			pPopup->GetMenuString(ID_POPUPMENU_EXIT, buf, MF_BYCOMMAND);
			pPopup->ModifyMenu(ID_POPUPMENU_EXIT, MF_BYCOMMAND | MF_MENUBARBREAK | MF_STRING, ID_POPUPMENU_EXIT, buf);
		}
		SetForegroundWindow();
		pPopup->TrackPopupMenu(TPM_RIGHTALIGN | TPM_LEFTBUTTON | TPM_RIGHTBUTTON, point.x, point.y, this);
	}
	else
	{
		pPopup->DeleteMenu(ID_POPUPMENU_PLAY, MF_BYCOMMAND);
		pPopup->DeleteMenu(ID_POPUPMENU_PAUSE, MF_BYCOMMAND);
		pPopup->DeleteMenu(ID_POPUPMENU_STOP, MF_BYCOMMAND);
		pPopup->DeleteMenu(ID_POPUPMENU_PREVIOUS, MF_BYCOMMAND);
		pPopup->DeleteMenu(ID_POPUPMENU_NEXT, MF_BYCOMMAND);
		pPopup->TrackPopupMenu(TPM_LEFTALIGN | TPM_LEFTBUTTON | TPM_RIGHTBUTTON, point.x, point.y, this);
	}
}


void CMainWnd::ShowTracksMenu(int MenuType, bool CurAlbOnly)
{
	if (m_PlaylistManager.GetSize() > 0 && m_PlaylistManager.GetSize() <= 50000)
	{
		POINT point;
		switch (MenuType)
		{
		case 0:
			point = CurMousePos; break;
		case 1:
			GetPosForMenu(&point); break;
		case 2:
			GetCursorPos(&point); break;
		}

		CMenu tmenu;
		tmenu.CreatePopupMenu();

		tmenu.DeleteMenu(0, MF_BYPOSITION);

		if (CurAlbOnly)
			BuildAlbumMenu(&tmenu);
		else
			BuildMenu(&tmenu);
		if (MenuType < 2)
		{
			SetForegroundWindow();
			tmenu.TrackPopupMenu(TPM_RIGHTALIGN | TPM_LEFTBUTTON | TPM_RIGHTBUTTON, point.x, point.y, this);
			PostMessage(WM_NULL, 0, 0);
		}
		else
			tmenu.TrackPopupMenu(TPM_RIGHTALIGN | TPM_LEFTBUTTON | TPM_RIGHTBUTTON, point.x, point.y, this);
		tmenu.DestroyMenu();
	}
}

void CMainWnd::echanger(int i, int j)
{
	CachedTag temp = m_PlaylistManager.Get(i);
	m_PlaylistManager.Add(m_PlaylistManager.Get(j), i);
	m_PlaylistManager.Add(temp, j);
	if(i==Current)
		Current=j;
	else
	if(j==Current)
		Current=i;
}

int CMainWnd::partition(int deb, int fin)
{
	int compt = deb;
	CString spivot = m_PlaylistManager.GetPlaylist(deb);
	CString tmp;
	int i;

	for (i = deb + 1; i <= fin; i++)
	{
		tmp = m_PlaylistManager.GetPlaylist(i);
		if (sup(spivot, tmp))
		{
			compt++;
			echanger(compt, i);
		}
	}
	echanger(compt, deb);
	return(compt);
}

void CMainWnd::SortRec(UINT L, UINT R, bool ForAut)
{
	if (L < R)
	{
		UINT pivot;

		pivot = partition(L, R);
		if (pivot > 0)
			SortRec(L, pivot - 1, ForAut);
		SortRec(pivot + 1, R, ForAut);
	}
}
void CMainWnd::Start2()
{
	stdIn = new CMultiDllInput(WAEmuWnd->m_hWnd);
	stdOut = new CMultiDllOutput(WAEmuWnd->m_hWnd);
	stdGen = new CMultiDllGen(WAEmuWnd->m_hWnd);

	stdIn->SetOutMod(stdOut->GetOut_Module());
	stdGen->Init();
	needToRestart = false;
	CurrentVolume = pVolume->GetCurrentVolume();

	LoadThePlaylist(false);
	if (AutOnStart)
		m_PlaylistManager.PerformAutomation();

	if (theApp.GetProfileInt(_T("Preferences"), _T("BarAtStartup"), 0) == 1)
	{
		if (SeekDialog == NULL)
			SeekDialog = SeekDialog->getInstance(IDD_SEEK, pMainFrame);
		SeekDialog->m_vol.SetPos(CurrentVolume);
		if (WAEmuWnd != NULL && WAEmuWnd->m_hWnd != NULL)
			WAEmuWnd->ShowWindow(SW_SHOW);
	}
	else
	{
		if (WAEmuWnd != NULL && WAEmuWnd->m_hWnd != NULL)
			WAEmuWnd->ShowWindow(SW_HIDE);
	}
	if (theApp.GetProfileInt(_T("Preferences"), _T("PlaylistAtStartup"), 0) == 1)
		PlaylistDlg = PlaylistDlg->getInstance(IDD_PLAYLIST);
	if (theApp.GetProfileInt(_T("Preferences"), _T("EQAtStartup"), 0) == 1)
		EqualizerDlg = EqualizerDlg->getInstance(IDD_EQUALIZER_DIALOG, pMainFrame);
	Created = true;

	if (autoUpdate)
	{
		CAutoUpdater updater;
		CAutoUpdater::ErrorType err = updater.CheckForUpdate(_T("http://systrayplay.free.fr/"));
		switch (err)
		{
		case CAutoUpdater::Success:
			MessageBox(m_Helper.MyLoadString(IDS_UPDATEOK), _T("STP"), MB_ICONINFORMATION | MB_OK);
			pMainFrame->OnPopupMenuExit();
			break;
		case CAutoUpdater::UpdateNotRequired:
			break;
		case CAutoUpdater::UpdateNotComplete:
			MessageBox(m_Helper.MyLoadString(IDS_UPDATEKO), _T("STP"), MB_ICONINFORMATION | MB_OK);
			break;
		case CAutoUpdater::NoExecutableVersion:
			MessageBox(m_Helper.MyLoadString(IDS_UPDATENOEXECUTABLEVERSION), _T("STP"), MB_ICONINFORMATION | MB_OK);
			break;
		default:
			MessageBox(m_Helper.MyLoadString(IDS_UPDATECONNECTFAILED), _T("STP"), MB_ICONINFORMATION | MB_OK);
			break;
		}
	}
	if (displayConsole)
	{
		pConsole->Popup();
	}
	firstPass = true;

#if _DEBUG
	OnPopupMenuSettingsAdvanced();
#endif
}
void CMainWnd::Start()
{
	TCHAR buf[MAX_PATH + 1];
	GetModuleFileName(theApp.m_hInstance, buf, MAX_PATH);
	exename.Format(_T("%s"), buf);

	StpIcon = (HICON)::LoadImage(theApp.m_hInstance, MAKEINTRESOURCE(IDI_STP), IMAGE_ICON, 32, 32, LR_DEFAULTCOLOR);
	StpIconSm = (HICON)::LoadImage(theApp.m_hInstance, MAKEINTRESOURCE(IDI_STP), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR);
	hMenuBitmap = (HBITMAP)::LoadImage(theApp.m_hInstance, MAKEINTRESOURCE(IDB_SYSTRAYPLAY), IMAGE_BITMAP, 20, 100, LR_DEFAULTCOLOR);
	m_imgList.Create(IDB_BITMAP1, 16, 0, RGB(192, 192, 192));
	LoadSkin();
	m_imgList.Add(PlayIcon);
	m_imgList.Add(PauseIcon);
	m_imgList.Add(StopIcon);
	m_imgList.Add(PrevIcon);
	m_imgList.Add(NextIcon);

	CMenuXP::SetMenuItemImage(ID_POPUPMENU_ABOUT, m_imgList, 0);
	CMenuXP::SetMenuItemImage(ID_POPUPMENU_OPEN_FILE, m_imgList, 1);
	CMenuXP::SetMenuItemImage(ID_PLAYLIST_SAVE, m_imgList, 2);
	CMenuXP::SetMenuItemImage(ID_POPUPMENU_OPENFOLDER, m_imgList, 4);
	CMenuXP::SetMenuItemImage(ID_POPUPMENU_PLAY, m_imgList, 5);
	CMenuXP::SetMenuItemImage(ID_POPUPMENU_PAUSE, m_imgList, 6);
	CMenuXP::SetMenuItemImage(ID_POPUPMENU_STOP, m_imgList, 7);
	CMenuXP::SetMenuItemImage(ID_POPUPMENU_PREVIOUS, m_imgList, 8);
	CMenuXP::SetMenuItemImage(ID_POPUPMENU_NEXT, m_imgList, 9);

	genres.Add(_T("Blues"));
	genres.Add(_T("Classic Rock"));
	genres.Add(_T("Country"));
	genres.Add(_T("Dance"));
	genres.Add(_T("Disco"));
	genres.Add(_T("Funk"));
	genres.Add(_T("Grunge"));
	genres.Add(_T("Hip-Hop"));
	genres.Add(_T("Jazz"));
	genres.Add(_T("Metal"));
	genres.Add(_T("New Age"));
	genres.Add(_T("Oldies"));
	genres.Add(_T("Other"));
	genres.Add(_T("Pop"));
	genres.Add(_T("R&B"));
	genres.Add(_T("Rap"));
	genres.Add(_T("Reggae"));
	genres.Add(_T("Rock"));
	genres.Add(_T("Techno"));
	genres.Add(_T("Industrial"));
	genres.Add(_T("Alternative"));
	genres.Add(_T("Ska"));
	genres.Add(_T("Death Metal"));
	genres.Add(_T("Pranks"));
	genres.Add(_T("Soundtrack"));
	genres.Add(_T("Euro-Techno"));
	genres.Add(_T("Ambient"));
	genres.Add(_T("Trip-Hop"));
	genres.Add(_T("Vocal"));
	genres.Add(_T("Jazz+Funk"));
	genres.Add(_T("Fusion"));
	genres.Add(_T("Trance"));
	genres.Add(_T("Classical"));
	genres.Add(_T("Instrumental"));
	genres.Add(_T("Acid"));
	genres.Add(_T("House"));
	genres.Add(_T("Game"));
	genres.Add(_T("Sound Clip"));
	genres.Add(_T("Gospel"));
	genres.Add(_T("Noise"));
	genres.Add(_T("Alt. Rock"));
	genres.Add(_T("Bass"));
	genres.Add(_T("Soul"));
	genres.Add(_T("Punk"));
	genres.Add(_T("Space"));
	genres.Add(_T("Meditative"));
	genres.Add(_T("Instrumental Pop"));
	genres.Add(_T("Instrumental Rock"));
	genres.Add(_T("Ethnic"));
	genres.Add(_T("Gothic"));
	genres.Add(_T("Darkwave"));
	genres.Add(_T("Techno-Industrial"));
	genres.Add(_T("Electronic"));
	genres.Add(_T("Pop-Folk"));
	genres.Add(_T("Eurodance"));
	genres.Add(_T("Dream"));
	genres.Add(_T("Southern Rock"));
	genres.Add(_T("Comedy"));
	genres.Add(_T("Cult"));
	genres.Add(_T("Gangsta Rap"));
	genres.Add(_T("Top 40"));
	genres.Add(_T("Christian Rap"));
	genres.Add(_T("Pop/Funk"));
	genres.Add(_T("Jungle"));
	genres.Add(_T("Native American"));
	genres.Add(_T("Cabaret"));
	genres.Add(_T("New Wave"));
	genres.Add(_T("Psychedelic"));
	genres.Add(_T("Rave"));
	genres.Add(_T("Showtunes"));
	genres.Add(_T("Trailer"));
	genres.Add(_T("Lo-Fi"));
	genres.Add(_T("Tribal"));
	genres.Add(_T("Acid Punk"));
	genres.Add(_T("Acid Jazz"));
	genres.Add(_T("Polka"));
	genres.Add(_T("Retro"));
	genres.Add(_T("Musical"));
	genres.Add(_T("Rock & Roll"));
	genres.Add(_T("Hard Rock"));
	genres.Add(_T("Folk"));
	genres.Add(_T("Folk/Rock"));
	genres.Add(_T("National Folk"));
	genres.Add(_T("Swing"));
	genres.Add(_T("Fast-Fusion"));
	genres.Add(_T("Bebob"));
	genres.Add(_T("Latin"));
	genres.Add(_T("Revival"));
	genres.Add(_T("Celtic"));
	genres.Add(_T("Bluegrass"));
	genres.Add(_T("Avantgarde"));
	genres.Add(_T("Gothic Rock"));
	genres.Add(_T("Progressive Rock"));
	genres.Add(_T("Psychedelic Rock"));
	genres.Add(_T("Symphonic Rock"));
	genres.Add(_T("Slow Rock"));
	genres.Add(_T("Big Band"));
	genres.Add(_T("Chorus"));
	genres.Add(_T("Easy Listening"));
	genres.Add(_T("Acoustic"));
	genres.Add(_T("Humour"));
	genres.Add(_T("Speech"));
	genres.Add(_T("Chanson"));
	genres.Add(_T("Opera"));
	genres.Add(_T("Chamber Music"));
	genres.Add(_T("Sonata"));
	genres.Add(_T("Symphony"));
	genres.Add(_T("Booty Bass"));
	genres.Add(_T("Primus"));
	genres.Add(_T("Porn Groove"));
	genres.Add(_T("Satire"));
	genres.Add(_T("Slow Jam"));
	genres.Add(_T("Club"));
	genres.Add(_T("Tango"));
	genres.Add(_T("Samba"));
	genres.Add(_T("Folklore"));
	genres.Add(_T("Ballad"));
	genres.Add(_T("Power Ballad"));
	genres.Add(_T("Rhythmic Soul"));
	genres.Add(_T("Freestyle"));
	genres.Add(_T("Duet"));
	genres.Add(_T("Punk Rock"));
	genres.Add(_T("Drum Solo"));
	genres.Add(_T("A Cappella"));
	genres.Add(_T("Euro-House"));
	genres.Add(_T("Dance Hall"));
	genres.Add(_T("Goa"));
	genres.Add(_T("Drum & Bass"));
	genres.Add(_T("Club-House"));
	genres.Add(_T("Hardcore"));
	genres.Add(_T("Terror"));
	genres.Add(_T("Indie"));
	genres.Add(_T("BritPop"));
	genres.Add(_T("Negerpunk"));
	genres.Add(_T("Polsk Punk"));
	genres.Add(_T("Beat"));
	genres.Add(_T("Christian Gangsta Rap"));
	genres.Add(_T("Heavy Metal"));
	genres.Add(_T("Black Metal"));
	genres.Add(_T("Crossover"));
	genres.Add(_T("Contemporary Christian"));
	genres.Add(_T("Christian Rock"));
	genres.Add(_T("Merengue"));
	genres.Add(_T("Salsa"));
	genres.Add(_T("Thrash Metal"));
	genres.Add(_T("Anime"));
	genres.Add(_T("JPop"));
	genres.Add(_T("Synthpop"));

	// initiating random list
	rnd = GetTickCount();
	{
		UINT i, j, k, l;
		for (i = 0; i < 50000; i++)
			rndlst[i] = i;
		srand((unsigned)time(NULL));
		for (i = 0; i < 75000; i++)
		{
			j = rand() % 50000;
			k = rand() % 50000;

			l = rndlst[j]; 
			rndlst[j] = rndlst[k]; 
			rndlst[k] = l;
		}
		rndpntr = rndlst;
	}

	FirstInMyRandom = 0xFFFFFFFF;

	TrackDurI = 0;

	ShutdownAfterEnd = 0;
	ExitAfterEnd = 0;

	TraySeeking = false;
	IconShown = true;

	lbd = rbd = false;
	lbs = rbs = 0;

	AddFromExp = false;

	CurMousePos.x = CurMousePos.y = 0;

	W2Ffile = theApp.GetProfileString(_T("Preferences"), _T("W2Ffile"), _T(""));
	W2Fplay = theApp.GetProfileString(_T("Preferences"), _T("W2Fplay"), _T("Now playing: %7"));
	W2Fpause = theApp.GetProfileString(_T("Preferences"), _T("W2Fpause"), _T("Now paused: %7"));
	W2Fstop = theApp.GetProfileString(_T("Preferences"), _T("W2Fstop"), _T("Now stopped: %7"));
	W2Fexit = theApp.GetProfileString(_T("Preferences"), _T("W2Fexit"), _T("STP is closed"));
	W2Foem = (int)theApp.GetProfileInt(_T("Preferences"), _T("W2Foem"), 0);
	W2Fdel = (int)theApp.GetProfileInt(_T("Preferences"), _T("W2Fdel"), 0);
	W2Fenabled = (int)theApp.GetProfileInt(_T("Preferences"), _T("W2Fenabled"), 0);

	AutSort = theApp.GetProfileInt(_T("Preferences"), _T("AutSort"), 0) == 1;
	AutRemove = theApp.GetProfileInt(_T("Preferences"), _T("AutRemove"), 0) == 1;
	AutRemovedup = theApp.GetProfileInt(_T("Preferences"), _T("AutRemovedup"), 0) == 1;
	AutRescan = theApp.GetProfileInt(_T("Preferences"), _T("AutRescan"), 0) == 1;
	AutPeriod = (int)theApp.GetProfileInt(_T("Preferences"), _T("AutPeriod"), 5);
	AutEvery = theApp.GetProfileInt(_T("Preferences"), _T("AutEvery"), 0) == 1;
	AutOnStart = theApp.GetProfileInt(_T("Preferences"), _T("AutOnStart"), 0) == 1;
	AutSaveset = theApp.GetProfileInt(_T("Preferences"), _T("AutSaveset"), 0) == 1;

	int AutDirsCnt = (int)theApp.GetProfileInt(_T("Preferences"), _T("AutDirsCnt"), 0);

	CString adtmp; int adc;
	for (adc = 0; adc < AutDirsCnt; adc++)
	{
		adtmp.Format(_T("%d"), adc + 1);
		adtmp = theApp.GetProfileString(_T("Preferences"), _T("AutDir") + adtmp, _T(""));
		if (adtmp != "")
			AutDirs.Add(adtmp);
	}

	pVolume = new CVolume();

	CurrentSkin = theApp.GetProfileString(_T("Preferences"), _T("CurrentSkin"), _T(""));

	RNTemplate = theApp.GetProfileString(_T("Preferences"), _T("RNTemplate"), _T("%1 - %3"));

	SMTracks = theApp.GetProfileInt(_T("Preferences"), _T("SMTracks"), 1) == 1;
	SMAlbum = theApp.GetProfileInt(_T("Preferences"), _T("SMAlbum"), 1) == 1;
	SMPrevNext = theApp.GetProfileInt(_T("Preferences"), _T("SMPrevNext"), 1) == 1;
#ifndef _UNICODE
	WAEmu = theApp.GetProfileInt(_T("Preferences"), _T("WAEmu"), 0) == 1;
#else
	WAEmu = false;
#endif

	DockSD = theApp.GetProfileInt(_T("Preferences"), _T("DockMinibar"), 0) == 1;

	Language = theApp.GetProfileString(_T("Preferences"), _T("Language"), _T(""));
	PressAndHold = (int)theApp.GetProfileInt(_T("Preferences"), _T("PressAndHold"), 2);

	ExtPlaylistFormat = (int)theApp.GetProfileInt(_T("Preferences"), _T("ExtPlaylistFormat"), 0);

	AlbumStop = theApp.GetProfileInt(_T("Preferences"), _T("AlbumStop"), 0) == 1;

	ShowingMode = (int)theApp.GetProfileInt(_T("Preferences"), _T("ShowingMode"), 0);
	m_PlaylistManager.SetShowingMode(ShowingMode);

	ShowGradientBar = theApp.GetProfileInt(_T("Preferences"), _T("ShowGradientBar"), 0) == 1;
	GraphicalButtons = theApp.GetProfileInt(_T("Preferences"), _T("GraphicalButtons"), 1) == 1;

	TPC = (int)theApp.GetProfileInt(_T("Preferences"), _T("TPC"), 10);

	ShowRemainingTime = (int)theApp.GetProfileInt(_T("Preferences"), _T("ShowRemainingTime"), 0);
	ShowTTT = theApp.GetProfileInt(_T("Preferences"), _T("ShowTTT"), 0) == 1;

	PlaylistOnTop = (int)theApp.GetProfileInt(_T("Preferences"), _T("PlaylistOnTop"), 0);
	EQOnTop = (int)theApp.GetProfileInt(_T("Preferences"), _T("EQOnTop"), 0);
	InfoOnTop = (int)theApp.GetProfileInt(_T("Preferences"), _T("InfoOnTop"), 0);
	NoClicks = (int)theApp.GetProfileInt(_T("Preferences"), _T("NoClicks"), 0);

	GapBtwTracks = (int)theApp.GetProfileInt(_T("Preferences"), _T("Gap"), 0);

	Clicks1 = theApp.GetProfileInt(_T("Preferences"), _T("Clicks1"), 1);
	Clicks2 = theApp.GetProfileInt(_T("Preferences"), _T("Clicks2"), 4);
	Clicks3 = theApp.GetProfileInt(_T("Preferences"), _T("Clicks3"), 3);
	Clicks4 = theApp.GetProfileInt(_T("Preferences"), _T("Clicks4"), 2);
	RClicks1 = theApp.GetProfileInt(_T("Preferences"), _T("RClicks1"), 10);
	RClicks2 = theApp.GetProfileInt(_T("Preferences"), _T("RClicks2"), 0);
	RClicks3 = theApp.GetProfileInt(_T("Preferences"), _T("RClicks3"), 0);
	RClicks4 = theApp.GetProfileInt(_T("Preferences"), _T("RClicks4"), 0);

	m_NPKey = theApp.GetProfileInt(_T("Preferences"), _T("NPKey"), 0);
	m_NPMod = theApp.GetProfileInt(_T("Preferences"), _T("NPMod"), 0);

	m_ExitKey = theApp.GetProfileInt(_T("Preferences"), _T("ExitKey"), 0);
	m_ExitMod = theApp.GetProfileInt(_T("Preferences"), _T("ExitMod"), 0);

	m_PlayKey = theApp.GetProfileInt(_T("Preferences"), _T("PlayKey"), 0);
	m_PlayMod = theApp.GetProfileInt(_T("Preferences"), _T("PlayMod"), 0);
	m_StopKey = theApp.GetProfileInt(_T("Preferences"), _T("StopKey"), 0);
	m_StopMod = theApp.GetProfileInt(_T("Preferences"), _T("StopMod"), 0);
	m_PrevKey = theApp.GetProfileInt(_T("Preferences"), _T("PrevKey"), 0);
	m_PrevMod = theApp.GetProfileInt(_T("Preferences"), _T("PrevMod"), 0);
	m_NextKey = theApp.GetProfileInt(_T("Preferences"), _T("NextKey"), 0);
	m_NextMod = theApp.GetProfileInt(_T("Preferences"), _T("NextMod"), 0);
	m_ForwKey = theApp.GetProfileInt(_T("Preferences"), _T("ForwKey"), 0);
	m_ForwMod = theApp.GetProfileInt(_T("Preferences"), _T("ForwMod"), 0);
	m_BackKey = theApp.GetProfileInt(_T("Preferences"), _T("BackKey"), 0);
	m_BackMod = theApp.GetProfileInt(_T("Preferences"), _T("BackMod"), 0);
	m_VUpKey = theApp.GetProfileInt(_T("Preferences"), _T("VUpKey"), 0);
	m_VUpMod = theApp.GetProfileInt(_T("Preferences"), _T("VUpMod"), 0);
	m_VDownKey = theApp.GetProfileInt(_T("Preferences"), _T("VDownKey"), 0);
	m_VDownMod = theApp.GetProfileInt(_T("Preferences"), _T("VDownMod"), 0);

	m_OpenKey1 = theApp.GetProfileInt(_T("Preferences"), _T("OpenKey1"), 0); 
	m_OpenMod1 = theApp.GetProfileInt(_T("Preferences"), _T("OpenMod1"), 0); 
	m_OpenKey2 = theApp.GetProfileInt(_T("Preferences"), _T("OpenKey2"), 0); 
	m_OpenMod2 = theApp.GetProfileInt(_T("Preferences"), _T("OpenMod2"), 0);
	m_OpenKey3 = theApp.GetProfileInt(_T("Preferences"), _T("OpenKey3"), 0); 
	m_OpenMod3 = theApp.GetProfileInt(_T("Preferences"), _T("OpenMod3"), 0);
	m_MinibarKey = theApp.GetProfileInt(_T("Preferences"), _T("MinibarKey"), 0);
	m_MinibarMod = theApp.GetProfileInt(_T("Preferences"), _T("MinibarMod"), 0);
	m_MP3InfoKey = theApp.GetProfileInt(_T("Preferences"), _T("MP3InfoKey"), 0);
	m_MP3InfoMod = theApp.GetProfileInt(_T("Preferences"), _T("MP3InfoMod"), 0);
	m_PlaylistKey = theApp.GetProfileInt(_T("Preferences"), _T("PlstKey"), 0);
	m_PlaylistMod = theApp.GetProfileInt(_T("Preferences"), _T("PlstMod"), 0);
	m_EqualizerKey = theApp.GetProfileInt(_T("Preferences"), _T("EqualizerKey"), 0);
	m_EqualizerMod = theApp.GetProfileInt(_T("Preferences"), _T("EqualizerMod"), 0);
	m_RepeatKey = theApp.GetProfileInt(_T("Preferences"), _T("RptKey"), 0);
	m_RepeatMod = theApp.GetProfileInt(_T("Preferences"), _T("RptMod"), 0);
	m_ShuffleKey = theApp.GetProfileInt(_T("Preferences"), _T("ShflKey"), 0);
	m_ShuffleMod = theApp.GetProfileInt(_T("Preferences"), _T("ShflMod"), 0);
	m_AlbummodeKey = theApp.GetProfileInt(_T("Preferences"), _T("AlbummodeKey"), 0);
	m_AlbummodeMod = theApp.GetProfileInt(_T("Preferences"), _T("AlbummodeMod"), 0);
	m_TrayIconKey = theApp.GetProfileInt(_T("Preferences"), _T("TrayIconKey"), 0);
	m_TrayIconMod = theApp.GetProfileInt(_T("Preferences"), _T("TrayIconMod"), 0);
	m_MenuKey = theApp.GetProfileInt(_T("Preferences"), _T("MenuKey"), 0);
	m_MenuMod = theApp.GetProfileInt(_T("Preferences"), _T("MenuMod"), 0);
	m_TracksKey = theApp.GetProfileInt(_T("Preferences"), _T("TracksKey"), 0);
	m_TracksMod = theApp.GetProfileInt(_T("Preferences"), _T("TracksMod"), 0);
	m_AlbumKey = theApp.GetProfileInt(_T("Preferences"), _T("AlbumKey"), 0);
	m_AlbumMod = theApp.GetProfileInt(_T("Preferences"), _T("AlbumMod"), 0);
	m_AutomationKey = theApp.GetProfileInt(_T("Preferences"), _T("AutomationKey"), 0);
	m_AutomationMod = theApp.GetProfileInt(_T("Preferences"), _T("AutomationMod"), 0);

	RegHotKeys();

	opening = false;

	Current = 0;

	InitDir = theApp.GetProfileString(_T("Preferences"), _T("InitDir"), _T("C:\\"));

	Recent[0] = theApp.GetProfileString(_T("Preferences"), _T("Recent0"), _T(""));
	Recent[1] = theApp.GetProfileString(_T("Preferences"), _T("Recent1"), _T(""));
	Recent[2] = theApp.GetProfileString(_T("Preferences"), _T("Recent2"), _T(""));
	Recent[3] = theApp.GetProfileString(_T("Preferences"), _T("Recent3"), _T(""));
	Recent[4] = theApp.GetProfileString(_T("Preferences"), _T("Recent4"), _T(""));
	Recent[5] = theApp.GetProfileString(_T("Preferences"), _T("Recent5"), _T(""));
	Recent[6] = theApp.GetProfileString(_T("Preferences"), _T("Recent6"), _T(""));
	Recent[7] = theApp.GetProfileString(_T("Preferences"), _T("Recent7"), _T(""));
	Recent[8] = theApp.GetProfileString(_T("Preferences"), _T("Recent8"), _T(""));
	Recent[9] = theApp.GetProfileString(_T("Preferences"), _T("Recent9"), _T(""));


	AutoPlay = (int)theApp.GetProfileInt(_T("Preferences"), _T("AutoPlay"), 1);
	RandomFirst = (int)theApp.GetProfileInt(_T("Preferences"), _T("RandomFirst"), 0);

	EQPreset = (int)theApp.GetProfileInt(_T("Preferences"), _T("EQPreset"), -1);

	SBDockR = theApp.GetProfileInt(_T("Preferences"), _T("SBDockR"), 0) == 1;
	SBDockB = theApp.GetProfileInt(_T("Preferences"), _T("SBDockB"), 0) == 1;
	SeekBarX = (int)theApp.GetProfileInt(_T("Preferences"), _T("SeekBarX"), -1000);
	SeekBarY = (int)theApp.GetProfileInt(_T("Preferences"), _T("SeekBarY"), -1000);
	SeekBarW = (int)theApp.GetProfileInt(_T("Preferences"), _T("SeekBarW"), -1000);
	EqualizerX = (int)theApp.GetProfileInt(_T("Preferences"), _T("EqualizerX"), -1000);
	EqualizerY = (int)theApp.GetProfileInt(_T("Preferences"), _T("EqualizerY"), -1000);
	PlaylistX = (int)theApp.GetProfileInt(_T("Preferences"), _T("PlaylistX"), -1000);
	PlaylistY = (int)theApp.GetProfileInt(_T("Preferences"), _T("PlaylistY"), -1000);
	PlaylistW = (int)theApp.GetProfileInt(_T("Preferences"), _T("PlaylistW"), -1000);
	PlaylistH = (int)theApp.GetProfileInt(_T("Preferences"), _T("PlaylistH"), -1000);
	InfoX = (int)theApp.GetProfileInt(_T("Preferences"), _T("InfoX"), -1000);
	InfoY = (int)theApp.GetProfileInt(_T("Preferences"), _T("InfoY"), -1000);

	m_Repeat = (int)theApp.GetProfileInt(_T("Preferences"), _T("Repeat"), 0);
	m_RepeatTrack = (int)theApp.GetProfileInt(_T("Preferences"), _T("RepeatTrack"), 0);
	ShuffleMode = (int)theApp.GetProfileInt(_T("Preferences"), _T("ShuffleMode"), 0);

	m_preamp = (int)theApp.GetProfileInt(_T("Preferences"), _T("Preamp"), 0);
	m_EEQ = (int)theApp.GetProfileInt(_T("Preferences"), _T("EnableEQ"), 0);

	m_s1 = (int)theApp.GetProfileInt(_T("Preferences"), _T("EQ1"), 0);
	m_s2 = (int)theApp.GetProfileInt(_T("Preferences"), _T("EQ2"), 0);
	m_s3 = (int)theApp.GetProfileInt(_T("Preferences"), _T("EQ3"), 0);
	m_s4 = (int)theApp.GetProfileInt(_T("Preferences"), _T("EQ4"), 0);
	m_s5 = (int)theApp.GetProfileInt(_T("Preferences"), _T("EQ5"), 0);
	m_s6 = (int)theApp.GetProfileInt(_T("Preferences"), _T("EQ6"), 0);
	m_s7 = (int)theApp.GetProfileInt(_T("Preferences"), _T("EQ7"), 0);
	m_s8 = (int)theApp.GetProfileInt(_T("Preferences"), _T("EQ8"), 0);
	m_s9 = (int)theApp.GetProfileInt(_T("Preferences"), _T("EQ9"), 0);
	m_s10 = (int)theApp.GetProfileInt(_T("Preferences"), _T("EQ10"), 0);

	showGradient = theApp.GetProfileInt(_T("MenuXP"), _T("ShowGradient"), 1) == 1;
	showToolTip = theApp.GetProfileInt(_T("Preferences"), _T("showToolTip"), 1) == 1;
	autoUpdate = theApp.GetProfileInt(_T("Preferences"), _T("autoUpdate"), 0) == 1;
	displayConsole = theApp.GetProfileInt(_T("Preferences"), _T("displayConsole"), 0) == 1;

	startColor = theApp.GetProfileInt(_T("MenuXP"), _T("GradientStart"), RGB(0xB4, 0xC4, 0xE4));
	endColor = theApp.GetProfileInt(_T("MenuXP"), _T("GradientEnd"), RGB(0xFF, 0xFF, 0xFF));
	toolTipText = theApp.GetProfileString(_T("Preferences"), _T("ToolTipText"), _T("<table>\\r\\n  <tr>\\r\\n    <td><icon idres=301></td>\\r\\n    <td><center><h2>%7</h2></center></td>\\r\\n    <td><right><a><icon idres=248 width=16 height=16 style=g hotstyle></a></right></td>\\r\\n  </tr>\\r\\n  <tr>\\r\\n    <td colspan=3><hr color=blue></td>\\r\\n  </tr>\\r\\n  <tr>\\r\\n    <td><bmp idres=138 mask style=d></td>\\r\\n    <td colspan=2>\\r\\n      <table>\\r\\n        <tr>\\r\\n          <td>Title :</td>\\r\\n          <td> %2</td>\\r\\n        </tr>\\r\\n        <tr>\\r\\n          <td>Artist :</td>\\r\\n          <td> %1</td>\\r\\n        </tr>\\r\\n        <tr><td>Album :</td>\\r\\n          <td> %3</td>\\r\\n        </tr>\\r\\n          <tr>\\r\\n          <td>Year :</td>\\r\\n          <td> %4</td>\\r\\n        </tr>\\r\\n        <tr>\\r\\n          <td>Comment :</td>\\r\\n          <td> %5</td>\\r\\n        </tr>\\r\\n        <tr>\\r\\n        <td>Length :</td>\\r\\n          <td> %6</td>\\r\\n        </tr>\\r\\n      </table>\\r\\n    </td>\\r\\n  </tr>\\r\\n</table>"));
	toolTipTime = theApp.GetProfileInt(_T("Preferences"), _T("ToolTipTime"), 5);


	int qq = theApp.GetProfileInt(_T("Preferences"), _T("Presets"), 4);

	CString buf1, buf2, buf3;
	for (int i = 0; i < qq; i++)
	{
		buf1.Format(_T("PresetName%d"), i);
		switch (i)
		{
		case 0:
			buf2 = _T("Classic");
			break;
		case 1:
			buf2 = _T("Loudness");
			break;
		case 2:
			buf2 = _T("Pop");
			break;
		case 3:
			buf2 = _T("Rock");
			break;
		default:
			buf2 = _T("");
			break;
		}
		buf2 = theApp.GetProfileString(_T("Preferences"), buf1, buf2);
		if (buf2 == "")
			continue;
		buf1.Format(_T("PresetEQ%d"), i);
		switch (i)
		{
		case 0:
			buf3 = _T("knjgjlnopqq");
			break;
		case 1:
			buf3 = _T("kqnlkkkmoqr");
			break;
		case 2:
			buf3 = _T("kkmnmlkkkkk");
			break;
		case 3:
			buf3 = _T("komkijlnopp");
			break;
		default:
			buf3 = _T("");
			break;
		}
		buf3 = theApp.GetProfileString(_T("Preferences"), buf1, buf3);
		if (buf3 == "")
			continue;
		presets.Add(buf2 + _T("\t") + buf3);
	}

	SetEQ();

	CurrentVolume = 0;

	nid.cbSize = sizeof(nid);
	nid.hWnd = this->m_hWnd;
	nid.uID = 1;
	nid.uFlags = NIF_ICON | NIF_TIP | NIF_MESSAGE;
	nid.uCallbackMessage = WM_ICON_NOTIFY;
	HICON m_hIcon = StopIcon;
	nid.hIcon = m_hIcon;
	_tcscpy(nid.szTip, _T("STP"));

	Shell_NotifyIcon(NIM_ADD, &nid);   
	XPLookNFeel = theApp.GetProfileInt(_T("Preferences"), _T("XPLookNFeel"), 0) == 1;

	SetTimer(REFRESH_WINDOW, 250, NULL);

	SetTimer(11, 1000, NULL);
	SetTimer(AUTOMATION_START, AutPeriod * 60000, NULL);
	startToolTip = false;
}

bool CMainWnd::sup(CString n1, CString n2)
{
	CString r1, r2;

	r1 = n1.Left(1 + n1.ReverseFind('\\'));
	r2 = n2.Left(1 + n2.ReverseFind('\\'));
	if (r1.Find(r2) == 0)
	{
		if (r1.GetLength() != r2.GetLength())
			return false;
		else
			return (n1.CompareNoCase(n2) > 0);
	}
	else
	{
		if (r2.Find(r1) == 0 && r1.GetLength() != r2.GetLength())
			return true;
		else
			return (n1.CompareNoCase(n2) > 0);
	}
}

bool CMainWnd::TryToEnd()
{
	if (ShutdownAfterEnd)
	{
		Finish();
		OSVERSIONINFO osinfo;
		osinfo.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
		::GetVersionEx(&osinfo);
		if (osinfo.dwPlatformId == VER_PLATFORM_WIN32_NT)
		{
			HANDLE hToken;
			TOKEN_PRIVILEGES tkp; 

			if (OpenProcessToken(GetCurrentProcess(), TOKEN_ADJUST_PRIVILEGES | TOKEN_QUERY, &hToken))
			{
				LookupPrivilegeValue(NULL, SE_SHUTDOWN_NAME, &tkp.Privileges[0].Luid); 
				tkp.PrivilegeCount = 1;
				tkp.Privileges[0].Attributes = SE_PRIVILEGE_ENABLED; 
				AdjustTokenPrivileges(hToken, FALSE, &tkp, 0, (PTOKEN_PRIVILEGES)NULL, 0); 
				if (GetLastError() == ERROR_SUCCESS)
					ExitWindowsEx(EWX_POWEROFF | EWX_FORCE, 0);
			}
		}
		else
			ExitWindowsEx(EWX_SHUTDOWN | EWX_FORCE, 0);
		DestroyWindow();  
		AfxPostQuitMessage(0);
		return true;
	}
	else if (ExitAfterEnd)
	{
		Finish();
		DestroyWindow();  
		AfxPostQuitMessage(0);
		return true;
	}
	return false;
}

void CMainWnd::UnregHotKeys()
{
	UnregisterHotKey(m_hWnd, 101);
	UnregisterHotKey(m_hWnd, 102);
	UnregisterHotKey(m_hWnd, 103);
	UnregisterHotKey(m_hWnd, 104);
	UnregisterHotKey(m_hWnd, 201);
	UnregisterHotKey(m_hWnd, 202);

	UnregisterHotKey(m_hWnd, 301);
	UnregisterHotKey(m_hWnd, 302);
	UnregisterHotKey(m_hWnd, 303);
	UnregisterHotKey(m_hWnd, 401);
	UnregisterHotKey(m_hWnd, 501);
	UnregisterHotKey(m_hWnd, 601);
	UnregisterHotKey(m_hWnd, 701);
	UnregisterHotKey(m_hWnd, 801);
	UnregisterHotKey(m_hWnd, 901);

	UnregisterHotKey(m_hWnd, 1001);
	UnregisterHotKey(m_hWnd, 1101);

	UnregisterHotKey(m_hWnd, 1201);

	UnregisterHotKey(m_hWnd, 1301);
	UnregisterHotKey(m_hWnd, 1302);

	UnregisterHotKey(m_hWnd, 1401);

	UnregisterHotKey(m_hWnd, 1501);
	UnregisterHotKey(m_hWnd, 1502);
	UnregisterHotKey(m_hWnd, 1503);

	UnregisterHotKey(m_hWnd, 1601);

	UnregisterHotKey(m_hWnd, 1701);
}

void CMainWnd::UpdMBTR(bool updtray)
{
	if (!pMainFrame->Created)
		return;
	if (m_PlaylistManager.GetSize() == 0)
	{
		SetWindowText(_T("STP MP3 Player"));
		nid.hIcon = StopIcon;
		if (IconShown)
		{
			_tcscpy(nid.szTip, _T("STP MP3 Player"));
			Shell_NotifyIcon(NIM_MODIFY, &(nid));
		}
		if (SeekDialog != NULL && SeekDialog->m_hWnd != NULL)
		{
			SeekDialog->CurrentSong = "";
			SeekDialog->m_bar.SetPos(0);
			SeekDialog->m_bar.RedrawWindow();
		}
		return;
	}
	TrackDurI = (int)(stdIn->GetLength() / 1000);

	if (TrackDurI <= 0)
		TrackDurI = 1;

	int p = 0;
	if (!stdIn->IsPaused())
	{
		nid.hIcon = PlayIcon;
		p = stdIn->GetOutputTime() / 1000;
	}
	else
	{
		nid.hIcon = PauseIcon;
		p = stdIn->GetOutputTime() / 1000;
	}
	if (p > TrackDurI)
		p = TrackDurI;
	if (SeekDialog != NULL && SeekDialog->m_hWnd != NULL)
	{
		if (!SeekDialog->ScrollFlag)
		{
			SeekDialog->m_bar.SetPos((int)((float)1000 * (float)p / (float)TrackDurI));
			if (ShowTTT)
				SeekDialog->CurrentTime.Format(_T("%02d:%02d/%02d:%02d"), ShowRemainingTime == 1 ? (int)((TrackDurI - p) / 60) : p / 60, ShowRemainingTime == 1 ? ((TrackDurI - p) % 60) : p % 60, TrackDurI / 60, TrackDurI % 60);
			else
				SeekDialog->CurrentTime.Format(_T("%02d:%02d"), ShowRemainingTime == 1 ? (int)((TrackDurI - p) / 60) : p / 60, ShowRemainingTime == 1 ? ((TrackDurI - p) % 60) : p % 60);
			SeekDialog->CurrentSong = m_PlaylistManager.GetTag(Current);
			if (ShowingMode != 2)
				SeekDialog->CurrentSong = SeekDialog->CurrentSong.Mid(0, SeekDialog->CurrentSong.ReverseFind('('));
			else
				SeekDialog->CurrentSong = m_PlaylistManager.GetFilename(Current);
			SeekDialog->m_bar.RedrawWindow();
		}
	}
	_stprintf(nid.szTip, _T("%s - %02d:%02d/%02d:%02d"), ReplaceAmpersand((m_PlaylistManager.GetFilename(Current).GetLength() <= 30 ? m_PlaylistManager.GetFilename(Current) : m_PlaylistManager.GetFilename(Current).Left(30) + "...")), ShowRemainingTime == 1 ? (int)((TrackDurI - p) / 60) : p / 60, ShowRemainingTime == 1 ? ((TrackDurI - p) % 60) : p % 60, TrackDurI / 60, TrackDurI % 60);

	if (WAEmuWnd != NULL && WAEmuWnd->m_hWnd != NULL)
	{
		CString tag, time, title;
		if (ShowTTT)
			time.Format(_T("%02d:%02d/%02d:%02d"), ShowRemainingTime == 1 ? (int)((TrackDurI - p) / 60) : p / 60, ShowRemainingTime == 1 ? ((TrackDurI - p) % 60) : p % 60, TrackDurI / 60, TrackDurI % 60);
		else
			time.Format(_T("%02d:%02d"), ShowRemainingTime == 1 ? (int)((TrackDurI - p) / 60) : p / 60, ShowRemainingTime == 1 ? ((TrackDurI - p) % 60) : p % 60);
		tag = m_PlaylistManager.GetTag(Current);
		if (ShowingMode != 2)
			tag = tag.Mid(0, tag.ReverseFind('('));
		else
			tag = m_PlaylistManager.GetFilename(Current);
#ifndef _UNICODE
		if (theApp.GetProfileInt(_T("Preferences"), _T("WAEmu"), 0) == 1)
			title.Format(_T("%s - Winamp"), tag);
		else
			title.Format(_T("%s %s - STPAppClass"), tag, time);
#else
		title.Format(_T("%s %s - STPAppClass"), tag, time);
#endif
		WAEmuWnd->SendMessage(WM_SETTEXT, 0, (LPARAM)title.GetString());
		ForNP = tag;
	}
	if (MP3InfoDlg != NULL && MP3InfoDlg->m_hWnd != NULL)
		MP3InfoDlg->ReadInfo();


	SetWindowText("STP: " + ForNP);

	if (IconShown && updtray)
		Shell_NotifyIcon(NIM_MODIFY, &(nid));
}

LRESULT CMainWnd::OnPostPlayerOver(UINT nFlags, LPARAM lp)
{
	TRACE(_T("next %d %d \n"), nFlags, lp);
	stdIn->SetPlayerMode(PLAYER_MODE_CLOSE);
	stdIn->Stop();
	GoNext();
	return 1;
}


LRESULT CMainWnd::WindowProc(UINT message, WPARAM wParam, LPARAM lParam)
{
	// custom messages

	//  	TRACE(_T("message %d %d %d \n"), message, wParam, lParam);
	//  if(message == WM_ICON_NOTIFY)
	//  	return IconNotify(wParam,lParam);

	//Rcupration du rglage de volume
	if (message == MM_MIXM_CONTROL_CHANGE)
	{
		CurrentVolume = pVolume->GetCurrentVolume();
		stdIn->SetVolume(CurrentVolume * 255 / 100);
		if (SeekDialog != NULL && SeekDialog->m_hWnd != NULL)
			SeekDialog->m_vol.SetPos(CurrentVolume);
		return TRUE;
	}
	if (message == WM_WA_IPC)
		return WAEmuWnd->IamWinamp(message, wParam, lParam);
	if (message == WM_HOTKEY)
	{
		TRACE(_T("WM_HOTKEY %d"), wParam);
		switch ((int)wParam)
		{
		case 101:
			//Play
			OnPopupMenuPlay();break;
		case 102:
			OnPopupMenuStop();break;
		case 103:
			OnPopupMenuPrevious();break;
		case 104:
			OnPopupMenuNext();break;
		case 201:
			UnregisterHotKey(m_hWnd, 201);
			CurrentVolume = pVolume->GetCurrentVolume();
			CurrentVolume += 1;
			if (CurrentVolume > 100)
				CurrentVolume = 100;
			pVolume->SetCurrentVolume(CurrentVolume);
			stdIn->SetVolume(CurrentVolume * 255 / 100);
			if (SeekDialog != NULL && SeekDialog->m_hWnd != NULL)
				SeekDialog->m_vol.SetPos(CurrentVolume);
			RegisterHotKey(m_hWnd, 201, m_VUpMod, m_VUpKey);
			break;
		case 202:
			UnregisterHotKey(m_hWnd, 202);
			CurrentVolume = pVolume->GetCurrentVolume();
			CurrentVolume -= 1;
			if (CurrentVolume < 0)
				CurrentVolume = 0;
			pVolume->SetCurrentVolume(CurrentVolume);
			stdIn->SetVolume(CurrentVolume * 255 / 100);
			if (SeekDialog != NULL && SeekDialog->m_hWnd != NULL)
				SeekDialog->m_vol.SetPos(CurrentVolume);
			RegisterHotKey(m_hWnd, 202, m_VDownMod, m_VDownKey);
			break;
		case 301:
			::SetForegroundWindow(m_hWnd);
			OnPopupMenuOpenFile();
			break;
		case 302:
			::SetForegroundWindow(m_hWnd);
			OnPopupMenuOpenFolder();
			break;
		case 303:
			OnPopupMenuOpenUrl();
			break;
		case 401:
			if (ShuffleMode == 1)
				ShuffleMode = 0;
			else
				ShuffleMode = 1;
			break;
		case 1601:
			if (AlbumStop)
				AlbumStop = false;
			else
				AlbumStop = true;
			break;
		case 501:
			::SetForegroundWindow(m_hWnd);
			OnPopupMenuSeek();
			break;
		case 601:
			::SetForegroundWindow(m_hWnd);
			OnPopupMenuEqualizer();
			break;
		case 701:
			if (m_Repeat == 0 && m_RepeatTrack == 0)
				m_Repeat = 1;
			else if (m_Repeat == 1 && m_RepeatTrack == 0)
			{
				m_Repeat = 0;m_RepeatTrack = 1;
			}
			else if (m_Repeat == 0 && m_RepeatTrack == 1)
				m_RepeatTrack = 0;
			break;
		case 801:
			::SetForegroundWindow(m_hWnd);
			OnPopupMenuPlaylistEditor();
			break;
		case 901:
			if (m_PlaylistManager.GetSize() > 0)
			{
				::SetForegroundWindow(m_hWnd);
				OnPopupMenuFileInfo();
			}
			break;
		case 1001:
			OnPopupMenuExit();
			break;
		case 1201:
			if (OpenClipboard())
			{
				if (EmptyClipboard())
				{
					HANDLE hData;
					hData = GlobalAlloc(GHND, ForNP.GetLength() + 5);
					LPTSTR pData;
					pData = (LPTSTR)GlobalLock(hData);
					lstrcpy((LPTSTR)pData, "np: " + ForNP);
					GlobalUnlock(hData);
					if (::SetClipboardData(CF_TEXT, hData) == NULL)
						GlobalFree(hData);
				}
				CloseClipboard();
			}
			break;
		case 1301:
			UnregisterHotKey(m_hWnd, 1301);
			if (m_PlaylistManager.GetSize() > 0 && stdIn->isSeekable())
				stdIn->SetOutputTime(stdIn->GetOutputTime() + 5);
			UpdMBTR();
			RegisterHotKey(m_hWnd, 1301, m_ForwMod, m_ForwKey);
			break;
		case 1302:
			UnregisterHotKey(m_hWnd, 1302);
			if (m_PlaylistManager.GetSize() > 0 && stdIn->isSeekable())
				stdIn->SetOutputTime(stdIn->GetOutputTime() - 5);
			UpdMBTR();
			RegisterHotKey(m_hWnd, 1302, m_BackMod, m_BackKey);
			break;
		case 1401:
			if (IconShown)
			{
				Shell_NotifyIcon(NIM_DELETE, &nid);
				IconShown = false;
			}
			else
			{
				Shell_NotifyIcon(NIM_ADD, &nid);
				IconShown = true;
			}
			break;
		case 1501:
			ShowPopup(1);
			break;
		case 1502:
			ShowTracksMenu(1);
			break;
		case 1503:
			ShowTracksMenu(1, true);
			break;
		case 1701:
			m_PlaylistManager.PerformAutomation();
			if (AutSaveset)
				SaveSettings();
			break;
		default:
			TRACE(_T("message non reu %d %d %d \n"), message, wParam, lParam);
		}
		return TRUE;
	}
	//TRACE(_T("message %d %d %d \n"), message, wParam, lParam);
	return CFrameWnd::WindowProc(message, wParam, lParam);
}

// Slectionneur d'action...

LRESULT CALLBACK CMainWnd::HookCallWndProc(int nCode, WPARAM wParam, LPARAM lParam)
{
	CWPSTRUCT cwps;
	static int x = 550;
	static int y = 550;
	static CMenu* sub = NULL;
	static CRect subRect(0, 0, 0, 0);

	if (nCode == HC_ACTION)
	{
		CopyMemory(&cwps, (LPVOID)lParam, sizeof(CWPSTRUCT));
		TCHAR szClass[128];
		GetClassName(cwps.hwnd, szClass, 127);
		//if(_tcscmp(szClass, _T("#32768"))==0)
		//  TRACE(_T("%s %d %d %d\n"),szClass,cwps.message,cwps.lParam,cwps.wParam);

		switch (cwps.message)
		{
		case 485:
			{
				if (_tcscmp(szClass, _T("#32768")) == 0)
				{
					CRect rect;
					//On rcuprer le menu que l'on est en train d'afficher
					HMENU hMenu = (HMENU)::SendMessage(cwps.hwnd, MN_GETHMENU, 0, 0);
					if (hMenu != NULL)
					{
						//  					TRACE(_T("%d %d\n"),CMenu::FromHandle(hMenu)->GetMenuItemCount(),CMenu::FromHandle(hMenu)->GetMenuItemID(cwps.wParam));
						CString buf;
						CMenu::FromHandle(hMenu)->GetMenuString(cwps.wParam, buf, MF_BYPOSITION);
						//S'il a un sous menu on mmorise sa position pour pouvoir coller le sous menu
						sub = CMenu::FromHandle(hMenu)->GetSubMenu(cwps.wParam);
						if (sub != NULL)
						{
							::GetMenuItemRect(cwps.hwnd, hMenu, cwps.wParam, &rect);
							//  						TRACE(_T("Nom : %s \n"),buf);
							//  						TRACE(_T("Dim %d %d %d %d\n"),rect.left,rect.top,rect.right, rect.bottom);
							subRect.left = rect.left;
							subRect.top = rect.top;
							subRect.right = rect.right;
							subRect.bottom = rect.bottom;
						}
						else
						{
							subRect.left = 0;
							subRect.top = 0;
							subRect.right = 0;
							subRect.bottom = 0;
						}
					}
					else
					{
						subRect.left = 0;
						subRect.top = 0;
						subRect.right = 0;
						subRect.bottom = 0;
					}
				}
				break;
			}
		case WM_WINDOWPOSCHANGED:
			{
				if (_tcscmp(szClass, _T("#32768")) == 0)
				{
					//  				TRACE(_T("subRect %d %d %d %d\n"),subRect.left,subRect.top,subRect.right, subRect.bottom);
					LPWINDOWPOS windowPos = (LPWINDOWPOS)cwps.lParam;
					//  				TRACE(_T("						%d %d %d %d\n"),windowPos->x, windowPos->y, windowPos->cx, windowPos->cy);
					if (windowPos->x + windowPos->cx < subRect.left)
						::SetWindowPos(cwps.hwnd, NULL, subRect.left - windowPos->cx, windowPos->y, windowPos->cx, windowPos->cy, SWP_NOSIZE | SWP_NOZORDER | SWP_NOACTIVATE);
				}
				break;
			}
		case WM_NCCREATE:
			{
				if (_tcscmp(szClass, _T("#32768")) == 0)
				{
					LPCREATESTRUCT createStruct = (LPCREATESTRUCT)cwps.lParam;
					//  				TRACE(_T("						%d %d %d %d\n"),createStruct->x,createStruct->y,createStruct->cx,createStruct->cy);
				}
				break;
			}
		case WM_NCCALCSIZE:
			{
				if (_tcscmp(szClass, _T("#32768")) == 0)
				{
					LPNCCALCSIZE_PARAMS ncCalcSize = (LPNCCALCSIZE_PARAMS)cwps.lParam;
					//  			TRACE(_T("						%d %d %d %d\n"),ncCalcSize->rgrc->left,ncCalcSize->rgrc->top,ncCalcSize->rgrc->right,ncCalcSize->rgrc->bottom);
				}
				break;
			}
		}
	}

	return CallNextHookEx((HHOOK)WH_CALLWNDPROC, nCode, wParam, lParam);
}